@@ -217,40 +217,6 @@ fGL element::GetVDWRadius(void) const
 	else return vdwr[atomic_number - 1];
 }
 
-/*
-const fGL * element::GetColor(prefs *user_prefs) const
-{
-	static fGL not_defined_color[3] = { 0.1, 0.1, 0.1 };
-	
-	if (atomic_number == NOT_DEFINED) return not_defined_color;
-
-	if (user_prefs == NULL)
-	  return color[atomic_number - 1];
-	else
-	  {
-	    char keybuffer[32];
-	    ostrstream fullkey(keybuffer, sizeof(keybuffer));
-	    fullkey << "ElementColor/" << atomic_number << ends;
-	    return user_prefs->ColorRGB(keybuffer, color[atomic_number - 1]);
-	  }
-}*/
-
-/*
-fGL element::GetVDWRadius(prefs *user_prefs) const
-{
-	if (atomic_number == NOT_DEFINED) return -1.0;
-
-	if (user_prefs == NULL)
-	  return vdwr[atomic_number - 1];
-	else
-	  {
-	    char keybuffer[32];
-	    ostrstream fullkey(keybuffer, sizeof(keybuffer));
-	    fullkey << "ElementMass/" << atomic_number - 1 << ends;
-	    return user_prefs->Double(keybuffer, vdwr[atomic_number - 1]);
-	  }
-}*/
-
 void element::operator++(void)
 {
 	if (++atomic_number > ELEMENT_SYMBOLS) atomic_number = 1;
@@ -414,6 +380,30 @@ void atom::SetCRD(i32s cs, fGL x, fGL y, fGL z)
 	}
 }
 
+bool atom::GetSelected() const
+{
+	if (flags & ATOMFLAG_USER_SELECTED) return true;
+	return false;
+}
+
+void atom::SetSelected(bool l)
+{
+	if(l) flags |= ATOMFLAG_USER_SELECTED;
+	else flags &= ~ATOMFLAG_USER_SELECTED;
+}
+
+bool atom::GetLocked() const
+{
+	if (flags & ATOMFLAG_USER_LOCKED) return true;
+	return false;
+}
+
+void atom::SetLocked(bool l)
+{
+	if(l) flags |= ATOMFLAG_USER_LOCKED;
+	else flags &= ~ATOMFLAG_USER_LOCKED;
+}
+
 // atoms will be sorted according to their molecule, chain and residue numbers.
 // sorting the container will then arrange molecules/chains/residues in continuous blocks.
 
