@@ -10,13 +10,13 @@
 #ifndef CONJGRAD_H
 #define CONJGRAD_H
 
+#include "libghemicaldefine.h"
 //#include "libghemicalconfig2.h"
 
 class conjugate_gradient;
 
 /*################################################################################################*/
 
-#include "libdefine.h"
 #include "typedef.h"
 
 #include <vector>
