@@ -14,8 +14,10 @@
 
 class setup1_mm;
 
-struct mm_bt1_data;		// saved distance results.
-struct mm_bt2_data;		// saved angle results.
+struct mm_bt1_data;	// saved distance results.
+struct mm_bt2_data;	// saved angle results.
+
+struct mm_c_dst;	// dst-constraint
 
 class eng1_mm;
 
@@ -54,8 +56,6 @@ class setup1_mm : virtual public setup
 	bool exceptions;
 	
 	friend class default_tables;
-	friend class setup_druid;	// FIXME:  for ghemical		// for BBB
-	friend class setup_dialog;	// FIXME:  for ghemical		// for HEAD
 	
 	public:
 	
@@ -63,6 +63,7 @@ class setup1_mm : virtual public setup
 	~setup1_mm(void);
 	
 	bool EnableExceptions(void) { return exceptions; }
+	void SetExceptions(bool e) { exceptions = e; }
 	
 	void UpdateAtomFlags(void);		// virtual
 	
@@ -93,6 +94,17 @@ struct mm_bt2_data		// saved angle results.
 	f64 dcsa[3][3];
 };
 
+struct mm_c_dst			// dst-constraint
+{
+	i32s atmi[2];
+	
+	f64 mindist; f64 minFC;
+	f64 maxdist; f64 maxFC;
+	
+	int bt1index;
+	bool skipNB;
+};
+
 /*################################################################################################*/
 
 /// A base engine class for molecular mechanics.
@@ -108,6 +120,10 @@ class eng1_mm : virtual public engine
 	vector<atom *> cr1; i32s * range_cr1;	// connectivity records...
 	vector<atom *> cr2; i32s * range_cr2;	// connectivity records...
 	
+	vector<mm_c_dst> c_dst_vector;
+	
+	bool do_virial;
+	
 	public:
 	
 	static const f64 fudgeLJ;	// how the 1-4 nonbonded interactions are modified...
@@ -128,7 +144,7 @@ class eng1_mm : virtual public engine
 	eng1_mm(setup *, i32u);
 	virtual ~eng1_mm(void);
 	
-	void Compute(i32u);	// virtual
+	void Compute(i32u, bool = false);	// virtual
 	
 	virtual i32s GetOrbitalCount(void) { return 0; }	// virtual
 	virtual f64 GetOrbitalEnergy(i32s) { return 0.0; }	// virtual
