@@ -9,17 +9,26 @@
 
 #include "eng1_mm_default.h"
 
+#include "v3d.h"
+
 #include "eng1_mm.h"
 #include "tab_mm_default.h"
 
 #include <algorithm>
-#include <strstream>
+#include <sstream>
 using namespace std;
 
 /*################################################################################################*/
 
-eng1_mm_default_bt::eng1_mm_default_bt(setup * p1, i32u p2) : engine(p1, p2), eng1_mm(p1, p2)
+eng1_mm_default_bt::eng1_mm_default_bt(setup * p1, i32u p2) :
+	engine(p1, p2),
+	eng1_mm(p1, p2)
 {
+	if (GetSetup()->GetModel()->GetConstD_count() > 0)
+	{
+		GetSetup()->GetModel()->WarningMessage(CONSTRAINTS_NOT_SUPPORTED);
+	}
+	
 	default_tables::GetInstance()->UpdateTypes(GetSetup());
 	default_tables::GetInstance()->UpdateCharges(GetSetup());	// for AMBER stuff, this affects types as well...
 	
@@ -496,10 +505,9 @@ if (newbt4.atmi[1] != n1) { cout << "OOPS THIS IS NOT WORKING!!!" << endl; exit(
 	i32s total_err = bt1_err + bt2_err + bt3_err + bt4_err;
 	if (total_err)
 	{
-		char mbuff1[256];
-		ostrstream str1(mbuff1, sizeof(mbuff1));
+		ostringstream str1;
 		str1 << "WARNING : there were " << total_err << " missing parameters in the bonded terms." << endl << ends;
-		GetSetup()->GetModel()->PrintToLog(mbuff1);
+		GetSetup()->GetModel()->PrintToLog(str1.str().c_str());
 	}
 }
 
@@ -679,6 +687,17 @@ if (atmtab[atmi[0]]->flags & ATOMFLAG_IS_SOLVENT_ATOM) E_solvent += t2b; else E_
 				
 				d1[l2g_mm[atmi[0]] * 3 + n2] += t2d;
 				d1[l2g_mm[atmi[1]] * 3 + n2] -= t2d;
+				
+				if (do_virial)
+				{
+				//f64 crdA = crd[l2g_mm[atmi[0]] * 3 + n2];
+				//f64 crdB = crd[l2g_mm[atmi[1]] * 3 + n2];
+				//f64 rAB = crdA - crdB;
+				///////////////////////////////////////////
+					f64 rAB = t1a[n2];
+					
+					virial[n2] -= rAB * t2d;	// F = -dE/dr
+				}
 			}
 		}
 	}
@@ -1040,7 +1059,10 @@ if (atmtab[atmi[0]]->flags & ATOMFLAG_IS_SOLVENT_ATOM) E_solvent += t2a; else E_
 
 /*################################################################################################*/
 
-eng1_mm_default_nbt_mbp::eng1_mm_default_nbt_mbp(setup * p1, i32u p2) : engine(p1, p2), eng1_mm(p1, p2), engine_mbp(p1, p2)
+eng1_mm_default_nbt_bp::eng1_mm_default_nbt_bp(setup * p1, i32u p2) :
+	engine(p1, p2),
+	eng1_mm(p1, p2),
+	engine_bp(p1, p2)
 {
 	atom ** atmtab = GetSetup()->GetMMAtoms();
 //	bond ** bndtab = GetSetup()->GetMMBonds();
@@ -1053,11 +1075,11 @@ eng1_mm_default_nbt_mbp::eng1_mm_default_nbt_mbp(setup * p1, i32u p2) : engine(p
 	bp_fc_solute = 5000.0;		// 50 kJ/(mol*^2) = 5000 kJ/(mol*(nm)^2)
 	bp_fc_solvent = 12500.0;	// 125 kJ/(mol*^2) = 12500 kJ/(mol*(nm)^2)
 	
-	if (ostr != NULL && use_simple_bp)
+	if (ostr != NULL && use_bp)
 	{
-		(* ostr) << "use_simple_bp ; ";
-		(* ostr) << bp_radius_solute << " " << bp_fc_solute << " ; ";
-		(* ostr) << bp_radius_solvent << " " << bp_fc_solvent << endl;
+		(* ostr) << "use_bp ; ";
+		(* ostr) << bp_rad_solute << " " << bp_fc_solute << " ; ";
+		(* ostr) << bp_rad_solvent << " " << bp_fc_solvent << endl;
 	}
 	
 	if (ostr != NULL) (* ostr) << "creating nbt1-terms: ";
@@ -1145,18 +1167,17 @@ eng1_mm_default_nbt_mbp::eng1_mm_default_nbt_mbp(setup * p1, i32u p2) : engine(p
 	i32s total_err = nbt1_err;
 	if (total_err)
 	{
-		char mbuff1[256];
-		ostrstream str1(mbuff1, sizeof(mbuff1));
+		ostringstream str1;
 		str1 << "WARNING : there were " << total_err << " missing parameters in the nonbonded terms." << endl << ends;
-		GetSetup()->GetModel()->PrintToLog(mbuff1);
+		GetSetup()->GetModel()->PrintToLog(str1.str().c_str());
 	}
 }
 
-eng1_mm_default_nbt_mbp::~eng1_mm_default_nbt_mbp(void)
+eng1_mm_default_nbt_bp::~eng1_mm_default_nbt_bp(void)
 {
 }
 
-void eng1_mm_default_nbt_mbp::ComputeNBT1(i32u p1)
+void eng1_mm_default_nbt_bp::ComputeNBT1(i32u p1)
 {
 	energy_nbt1a = 0.0;
 	energy_nbt1b = 0.0;
@@ -1169,24 +1190,24 @@ void eng1_mm_default_nbt_mbp::ComputeNBT1(i32u p1)
 	// an additional pass for the boundary potential (optional).
 	// an additional pass for the boundary potential (optional).
 	
-	if (use_simple_bp)
+	if (use_bp)
 	{
 		if (nd_eval != NULL) nd_eval->AddCycle();
 		for (i32s n1 = 0;n1 < GetSetup()->GetMMAtomCount();n1++)
 		{
-			f64 radius = bp_radius_solute;
+			f64 radius = bp_rad_solute;
 			f64 fc = bp_fc_solute;
 			
 			if (atmtab[n1]->flags & ATOMFLAG_IS_SOLVENT_ATOM)
 			{
-				radius = bp_radius_solvent;
+				radius = bp_rad_solvent;
 				fc = bp_fc_solvent;
 			}
 			
 			f64 t1a[3]; f64 t1b = 0.0;
 			for (i32s n2 = 0;n2 < 3;n2++)
 			{
-				f64 t9a = bp_center[n2];
+				f64 t9a = 0.0;//bp_center[n2];
 				f64 t9b = crd[l2g_mm[n1] * 3 + n2];
 				
 				t1a[n2] = t9a - t9b;
@@ -1312,17 +1333,20 @@ else E_solusolv += tote;
 
 /*################################################################################################*/
 
-eng1_mm_default_nbt_mim::eng1_mm_default_nbt_mim(setup * p1, i32u p2) : engine(p1, p2), eng1_mm(p1, p2), engine_pbc(p1, p2)
+eng1_mm_default_nbt_mim::eng1_mm_default_nbt_mim(setup * p1, i32u p2) :
+	engine(p1, p2),
+	eng1_mm(p1, p2),
+	engine_pbc(p1, p2)
 {
-	fGL mindim = GetSetup()->GetModel()->periodic_box_HALFdim[0];
-	if (GetSetup()->GetModel()->periodic_box_HALFdim[1] < mindim) mindim = GetSetup()->GetModel()->periodic_box_HALFdim[1];
-	if (GetSetup()->GetModel()->periodic_box_HALFdim[2] < mindim) mindim = GetSetup()->GetModel()->periodic_box_HALFdim[2];
+	fGL mindim = box_HALFdim[0];
+	if (box_HALFdim[1] < mindim) mindim = box_HALFdim[1];
+	if (box_HALFdim[2] < mindim) mindim = box_HALFdim[2];
 	
 	sw1 = 0.6; if (sw1 < (mindim - 0.4)) sw1 = mindim - 0.4;	// will trigger if boxdim < 2.0 nm!!!
 	sw2 = shft1 = mindim - 0.2;
 	
 	limit = mindim;
-	update = true;
+	update_neighbor_list = true;
 	
 	// calculate the actual values...
 	
@@ -1352,7 +1376,7 @@ void eng1_mm_default_nbt_mim::ComputeNBT1(i32u p1)
 	
 	atom ** atmtab = GetSetup()->GetMMAtoms();
 	
-	if (update) UpdateTerms();
+	if (update_neighbor_list) UpdateTerms();
 	
 	// the nonbonded terms begin...
 	// the nonbonded terms begin...
@@ -1366,17 +1390,56 @@ void eng1_mm_default_nbt_mim::ComputeNBT1(i32u p1)
 		for (i32s n2 = 0;n2 < 3;n2++)
 		{
 			f64 t2a = crd[l2g_mm[atmi[0]] * 3 + n2];
+			// ^^^^^ either at primary cell OR at 1st neighbor cell (1 of 26).
+	if (t2a < -box_HALFdim[n2])
+	{
+		t2a += 2.0 * box_HALFdim[n2];
+		if (t2a < -box_HALFdim[n2])
+		{
+			cout << "PBC failed ; a-" << endl;
+			exit(EXIT_FAILURE);
+		}
+	}
+	else if (t2a > +box_HALFdim[n2])
+	{
+		t2a -= 2.0 * box_HALFdim[n2];
+		if (t2a > +box_HALFdim[n2])
+		{
+			cout << "PBC failed ; a+" << endl;
+			exit(EXIT_FAILURE);
+		}
+	}
+			
 			f64 t2b = crd[l2g_mm[atmi[1]] * 3 + n2];
+			// ^^^^^ either at primary cell OR at 1st neighbor cell (1 of 26).
+	if (t2b < -box_HALFdim[n2])
+	{
+		t2b += 2.0 * box_HALFdim[n2];
+		if (t2b < -box_HALFdim[n2])
+		{
+			cout << "PBC failed ; b-" << endl;
+			exit(EXIT_FAILURE);
+		}
+	}
+	else if (t2b > +box_HALFdim[n2])
+	{
+		t2b -= 2.0 * box_HALFdim[n2];
+		if (t2b > +box_HALFdim[n2])
+		{
+			cout << "PBC failed ; b+" << endl;
+			exit(EXIT_FAILURE);
+		}
+	}
 			
 			t1a[n2] = t2a - t2b;
 			
-			if (t1a[n2] < -GetSetup()->GetModel()->periodic_box_HALFdim[n2])
+			if (t1a[n2] < -box_HALFdim[n2])
 			{
-				t1a[n2] += 2.0 * GetSetup()->GetModel()->periodic_box_HALFdim[n2];
+				t1a[n2] += 2.0 * box_HALFdim[n2];
 			}
-			else if (t1a[n2] > +GetSetup()->GetModel()->periodic_box_HALFdim[n2])
+			else if (t1a[n2] > +box_HALFdim[n2])
 			{
-				t1a[n2] -= 2.0 * GetSetup()->GetModel()->periodic_box_HALFdim[n2];
+				t1a[n2] -= 2.0 * box_HALFdim[n2];
 			}
 			
 			t1b += t1a[n2] * t1a[n2];
@@ -1472,6 +1535,17 @@ else E_solusolv += tote;
 				
 				d1[l2g_mm[atmi[0]] * 3 + n2] += t9d;
 				d1[l2g_mm[atmi[1]] * 3 + n2] -= t9d;
+				
+				if (do_virial)
+				{
+				//f64 crdA = crd[l2g_mm[atmi[0]] * 3 + n2];	eiky...
+				//f64 crdB = crd[l2g_mm[atmi[1]] * 3 + n2];	eiky...
+				//f64 rAB = crdA - crdB;			eiky...
+				//////////////////////////////////////////////////////////
+					f64 rAB = t1a[n2];
+					
+					virial[n2] -= rAB * t9d;	// F = -dE/dr
+				}
 			}
 		}
 	}
@@ -1479,12 +1553,13 @@ else E_solusolv += tote;
 
 void eng1_mm_default_nbt_mim::UpdateTerms(void)
 {
+	update_neighbor_list = false;	// processed...
+	
 	atom ** atmtab = GetSetup()->GetMMAtoms();
 //	bond ** bndtab = GetSetup()->GetMMBonds();
 	
 	ostream * ostr = default_tables::GetInstance()->ostr;
 	
-	update = false;
 	nbt1_vector.resize(0);
 	
 	if (ostr != NULL) (* ostr) << "creating nbt1-terms: ";
@@ -1507,17 +1582,56 @@ void eng1_mm_default_nbt_mim::UpdateTerms(void)
 				for (i32s n1 = 0;n1 < 3;n1++)
 				{
 					f64 t2a = crd[l2g_mm[ind1] * 3 + n1];
+					// ^^^^^ either at primary cell OR at 1st neighbor cell (1 of 26).
+	if (t2a < -box_HALFdim[n1])
+	{
+		t2a += 2.0 * box_HALFdim[n1];
+		if (t2a < -box_HALFdim[n1])
+		{
+			cout << "PBC failed ; a-" << endl;
+			exit(EXIT_FAILURE);
+		}
+	}
+	else if (t2a > +box_HALFdim[n1])
+	{
+		t2a -= 2.0 * box_HALFdim[n1];
+		if (t2a > +box_HALFdim[n1])
+		{
+			cout << "PBC failed ; a+" << endl;
+			exit(EXIT_FAILURE);
+		}
+	}
+					
 					f64 t2b = crd[l2g_mm[ind2] * 3 + n1];
+					// ^^^^^ either at primary cell OR at 1st neighbor cell (1 of 26).
+	if (t2b < -box_HALFdim[n1])
+	{
+		t2b += 2.0 * box_HALFdim[n1];
+		if (t2b < -box_HALFdim[n1])
+		{
+			cout << "PBC failed ; b-" << endl;
+			exit(EXIT_FAILURE);
+		}
+	}
+	else if (t2b > +box_HALFdim[n1])
+	{
+		t2b -= 2.0 * box_HALFdim[n1];
+		if (t2b > +box_HALFdim[n1])
+		{
+			cout << "PBC failed ; b+" << endl;
+			exit(EXIT_FAILURE);
+		}
+	}
 					
 					t1a = t2a - t2b;
 					
-					if (t1a < -GetSetup()->GetModel()->periodic_box_HALFdim[n1])
+					if (t1a < -box_HALFdim[n1])
 					{
-						t1a += 2.0 * GetSetup()->GetModel()->periodic_box_HALFdim[n1];
+						t1a += 2.0 * box_HALFdim[n1];
 					}
-					else if (t1a > +GetSetup()->GetModel()->periodic_box_HALFdim[n1])
+					else if (t1a > +box_HALFdim[n1])
 					{
-						t1a -= 2.0 * GetSetup()->GetModel()->periodic_box_HALFdim[n1];
+						t1a -= 2.0 * box_HALFdim[n1];
 					}
 					
 					t1b += t1a * t1a;
@@ -1545,8 +1659,8 @@ void eng1_mm_default_nbt_mim::UpdateTerms(void)
 				}
 				else
 				{
-					// see also eng1_mm_default_nbt_mbp ctor ; should be the same!!!
-					// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+					// see also eng1_mm_default_nbt_bp ctor ; should be the same!!!
+					// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 					
 					bool errors = false;
 					const default_at * at;
@@ -1594,28 +1708,34 @@ void eng1_mm_default_nbt_mim::UpdateTerms(void)
 	i32s total_err = nbt1_err;
 	if (total_err)
 	{
-		char mbuff1[256];
-		ostrstream str1(mbuff1, sizeof(mbuff1));
+		ostringstream str1;
 		str1 << "WARNING : there were " << total_err << " missing parameters in the nonbonded terms." << endl << ends;
-		GetSetup()->GetModel()->PrintToLog(mbuff1);
+		GetSetup()->GetModel()->PrintToLog(str1.str().c_str());
 	}
 }
 
 /*################################################################################################*/
 
-eng1_mm_default_mbp::eng1_mm_default_mbp(setup * p1, i32u p2) : engine(p1, p2), eng1_mm(p1, p2),
-	engine_mbp(p1, p2), eng1_mm_default_bt(p1, p2), eng1_mm_default_nbt_mbp(p1, p2)
+eng1_mm_default_bp::eng1_mm_default_bp(setup * p1, i32u p2) :
+	engine(p1, p2),
+	eng1_mm(p1, p2),
+	engine_bp(p1, p2),
+	eng1_mm_default_bt(p1, p2),
+	eng1_mm_default_nbt_bp(p1, p2)
 {
 }
 
-eng1_mm_default_mbp::~eng1_mm_default_mbp(void)
+eng1_mm_default_bp::~eng1_mm_default_bp(void)
 {
 }
 
 /*################################################################################################*/
 
-eng1_mm_default_mim::eng1_mm_default_mim(setup * p1, i32u p2) : engine(p1, p2), eng1_mm(p1, p2),
-	eng1_mm_default_bt(p1, p2), eng1_mm_default_nbt_mim(p1, p2)
+eng1_mm_default_mim::eng1_mm_default_mim(setup * p1, i32u p2) :
+	engine(p1, p2),
+	eng1_mm(p1, p2),
+	eng1_mm_default_bt(p1, p2),
+	eng1_mm_default_nbt_mim(p1, p2)
 {
 }
 
