@@ -9,9 +9,11 @@
 
 #include "eng1_mm_prmfit.h"
 
+#include "v3d.h"
+
 #include <iomanip>
 #include <fstream>
-#include <strstream>
+#include <sstream>
 #include <algorithm>
 using namespace std;
 
@@ -19,6 +21,11 @@ using namespace std;
 
 eng1_mm_prmfit::eng1_mm_prmfit(setup * p1, i32u p2, prmfit_tables & tab) : engine(p1, p2), eng1_mm(p1, p2)
 {
+	if (GetSetup()->GetModel()->GetConstD_count() > 0)
+	{
+		GetSetup()->GetModel()->WarningMessage(CONSTRAINTS_NOT_SUPPORTED);
+	}
+	
 	tab.UpdateTypes(GetSetup());
 	tab.UpdateCharges(GetSetup());
 	
@@ -456,10 +463,9 @@ if (newbt4.atmi[1] != n1) { cout << "OOPS THIS IS NOT WORKING!!!" << endl; exit(
 	total_err = bt1_err + bt2_err + bt3_err + bt4_err;
 	if (total_err)
 	{
-		char mbuff1[256];
-		ostrstream str1(mbuff1, sizeof(mbuff1));
+		ostringstream str1;
 		str1 << "WARNING : there were " << total_err << " missing parameters in the bonded terms." << endl << ends;
-		GetSetup()->GetModel()->PrintToLog(mbuff1);
+		GetSetup()->GetModel()->PrintToLog(str1.str().c_str());
 	}
 	
 /*##############################################*/
@@ -542,10 +548,9 @@ if (newbt4.atmi[1] != n1) { cout << "OOPS THIS IS NOT WORKING!!!" << endl; exit(
 	total_err = nbt1_err;
 	if (total_err)
 	{
-		char mbuff1[256];
-		ostrstream str1(mbuff1, sizeof(mbuff1));
+		ostringstream str1;
 		str1 << "WARNING : there were " << total_err << " missing parameters in the nonbonded terms." << endl << ends;
-		GetSetup()->GetModel()->PrintToLog(mbuff1);
+		GetSetup()->GetModel()->PrintToLog(str1.str().c_str());
 	}
 }
 
