@@ -9,16 +9,20 @@
 
 #include "eng1_mm_tripos52.h"
 
+#include "v3d.h"
+
 #include "eng1_mm.h"
 #include "tab_mm_tripos52.h"
 
 #include <algorithm>
-#include <strstream>
+#include <sstream>
 using namespace std;
 
 /*################################################################################################*/
 
-eng1_mm_tripos52_bt::eng1_mm_tripos52_bt(setup * p1, i32u p2) : engine(p1, p2), eng1_mm(p1, p2)
+eng1_mm_tripos52_bt::eng1_mm_tripos52_bt(setup * p1, i32u p2) :
+	engine(p1, p2),
+	eng1_mm(p1, p2)
 {
 	tripos52_tables::GetInstance()->UpdateTypes(GetSetup());
 	tripos52_tables::GetInstance()->UpdateCharges(GetSetup());
@@ -54,6 +58,24 @@ eng1_mm_tripos52_bt::eng1_mm_tripos52_bt(setup * p1, i32u p2) : engine(p1, p2),
 		(* it1).tmp_bt1_index = bt1_vector.size();	// the bond objects are modified here!!!!!
 		
 		bt1_err += !tripos52_tables::GetInstance()->Init(this, & newbt1, bt);
+		
+		// if any of the distance constraints match with a bond,
+		// then set the force constant of the bond to zero...
+		
+		for (i32u cind = 0;cind < c_dst_vector.size();cind++)
+		{
+			bool match = false;
+			if (c_dst_vector[cind].atmi[0] == newbt1.atmi[0] && c_dst_vector[cind].atmi[1] == newbt1.atmi[1]) match = true;
+			if (c_dst_vector[cind].atmi[0] == newbt1.atmi[1] && c_dst_vector[cind].atmi[1] == newbt1.atmi[0]) match = true;
+			
+			if (match)
+			{
+				c_dst_vector[cind].bt1index = bt1_vector.size();
+				newbt1.fc = 0.0;
+				break;
+			}
+		}
+		
 		bt1_vector.push_back(newbt1);
 	}
 	
@@ -285,10 +307,9 @@ eng1_mm_tripos52_bt::eng1_mm_tripos52_bt(setup * p1, i32u p2) : engine(p1, p2),
 	i32s total_err = bt1_err + bt2_err + bt3_err;
 	if (total_err)
 	{
-		char mbuff1[256];
-		ostrstream str1(mbuff1, sizeof(mbuff1));
+		ostringstream str1;
 		str1 << "WARNING : there were " << total_err << " missing parameters in the bonded terms." << endl << ends;
-		mdl->PrintToLog(mbuff1);
+		mdl->PrintToLog(str1.str().c_str());
 	}
 }
 
@@ -513,6 +534,62 @@ if (atmtab[atmi[0]]->flags & ATOMFLAG_IS_SOLVENT_ATOM) E_solvent += t2b; else E_
 			}
 		}
 	}
+	
+	// process the distance constraints.
+	
+	for (i32s n1 = 0;n1 < (i32s) c_dst_vector.size();n1++)
+	{
+		i32s * atmi = c_dst_vector[n1].atmi;
+		
+		f64 t1a[3]; f64 t1b = 0.0;
+		for (i32s n2 = 0;n2 < 3;n2++)
+		{
+			f64 t9a = crd[l2g_mm[atmi[0]] * 3 + n2];
+			f64 t9b = crd[l2g_mm[atmi[1]] * 3 + n2];
+			
+			t1a[n2] = t9a - t9b;
+			t1b += t1a[n2] * t1a[n2];
+		}
+		
+		f64 t1c = sqrt(t1b);
+		
+		f64 lim; f64 fc;
+		f64 t2b = 0.0; f64 t2c = 0.0;
+		
+		// f = a(x-b)^2
+		// df/dx = 2a(x-b)
+		
+		lim = c_dst_vector[n1].mindist;
+		fc = c_dst_vector[n1].minFC;
+		if (lim > 0.0 && t1c < lim)
+		{
+			f64 t2a = t1c - lim;
+			t2b = fc * t2a * t2a;
+			t2c = 2.0 * fc * t2a;
+		}
+		
+		lim = c_dst_vector[n1].maxdist;
+		fc = c_dst_vector[n1].maxFC;
+		if (lim > 0.0 && t1c > lim)
+		{
+			f64 t2a = t1c - lim;
+			t2b = fc * t2a * t2a;
+			t2c = 2.0 * fc * t2a;
+		}
+		
+		energy_bt1 += t2b;
+		
+		if (p1 > 0)
+		{
+			for (i32s n2 = 0;n2 < 3;n2++)
+			{
+				f64 t2d = (t1a[n2] / t1c) * t2c;
+				
+				d1[l2g_mm[atmi[0]] * 3 + n2] += t2d;
+				d1[l2g_mm[atmi[1]] * 3 + n2] -= t2d;
+			}
+		}
+	}
 }
 
 void eng1_mm_tripos52_bt::ComputeBT2(i32u p1)
@@ -766,7 +843,10 @@ void eng1_mm_tripos52_bt::ComputeBT4(i32u)
 
 /*################################################################################################*/
 
-eng1_mm_tripos52_nbt_mbp::eng1_mm_tripos52_nbt_mbp(setup * p1, i32u p2) : engine(p1, p2), eng1_mm(p1, p2), engine_mbp(p1, p2)
+eng1_mm_tripos52_nbt_bp::eng1_mm_tripos52_nbt_bp(setup * p1, i32u p2) :
+	engine(p1, p2),
+	eng1_mm(p1, p2),
+	engine_bp(p1, p2)
 {
 	model * mdl = GetSetup()->GetModel();
 	atom ** atmtab = GetSetup()->GetMMAtoms();
@@ -781,11 +861,11 @@ eng1_mm_tripos52_nbt_mbp::eng1_mm_tripos52_nbt_mbp(setup * p1, i32u p2) : engine
 	bp_fc_solute = 5000.0;		// 50 kJ/(mol*^2) = 5000 kJ/(mol*(nm)^2)
 	bp_fc_solvent = 12500.0;	// 125 kJ/(mol*^2) = 12500 kJ/(mol*(nm)^2)
 	
-	if (ostr != NULL && use_simple_bp)
+	if (ostr != NULL && use_bp)
 	{
-		(* ostr) << "use_simple_bp ; ";
-		(* ostr) << bp_radius_solute << " " << bp_fc_solute << " ; ";
-		(* ostr) << bp_radius_solvent << " " << bp_fc_solvent << endl;
+		(* ostr) << "use_bp ; ";
+		(* ostr) << bp_rad_solute << " " << bp_fc_solute << " ; ";
+		(* ostr) << bp_rad_solvent << " " << bp_fc_solvent << endl;
 	}
 	
 	if (ostr != NULL) (* ostr) << "creating nbt1-terms: ";
@@ -853,8 +933,29 @@ eng1_mm_tripos52_nbt_mbp::eng1_mm_tripos52_nbt_mbp(setup * p1, i32u p2) : engine
 				newnbt1.atmi[0] = ind1;		// this is a local index...
 				newnbt1.atmi[1] = ind2;		// this is a local index...
 				
-				nbt1_err += !tripos52_tables::GetInstance()->Init(this, & newnbt1, is14);
-				nbt1_vector.push_back(newnbt1);
+				bool add_term = true;
+				
+	// check from the distance constraints if there is a request to skip this term.
+	
+	for (i32u cind = 0;cind < c_dst_vector.size();cind++)
+	{
+		if (!c_dst_vector[cind].skipNB) continue;
+		
+		bool match = false;
+		if (c_dst_vector[cind].atmi[0] == newnbt1.atmi[0] && c_dst_vector[cind].atmi[1] == newnbt1.atmi[1]) match = true;
+		if (c_dst_vector[cind].atmi[0] == newnbt1.atmi[1] && c_dst_vector[cind].atmi[1] == newnbt1.atmi[0]) match = true;
+		
+		if (match)
+		{
+			add_term = false;
+		}
+	}
+				
+				if (add_term)
+				{
+					nbt1_err += !tripos52_tables::GetInstance()->Init(this, & newnbt1, is14);
+					nbt1_vector.push_back(newnbt1);
+				}
 			}
 		}
 	}
@@ -864,18 +965,17 @@ eng1_mm_tripos52_nbt_mbp::eng1_mm_tripos52_nbt_mbp(setup * p1, i32u p2) : engine
 	i32s total_err = nbt1_err;
 	if (total_err)
 	{
-		char mbuff1[256];
-		ostrstream str1(mbuff1, sizeof(mbuff1));
+		ostringstream str1;
 		str1 << "WARNING : there were " << total_err << " missing parameters in the nonbonded terms." << endl << ends;
-		mdl->PrintToLog(mbuff1);
+		mdl->PrintToLog(str1.str().c_str());
 	}
 }
 
-eng1_mm_tripos52_nbt_mbp::~eng1_mm_tripos52_nbt_mbp(void)
+eng1_mm_tripos52_nbt_bp::~eng1_mm_tripos52_nbt_bp(void)
 {
 }
 
-void eng1_mm_tripos52_nbt_mbp::ComputeNBT1(i32u p1)
+void eng1_mm_tripos52_nbt_bp::ComputeNBT1(i32u p1)
 {
 	energy_nbt1a = 0.0;
 	energy_nbt1b = 0.0;
@@ -888,24 +988,24 @@ void eng1_mm_tripos52_nbt_mbp::ComputeNBT1(i32u p1)
 	// an additional pass for the boundary potential (optional).
 	// an additional pass for the boundary potential (optional).
 	
-	if (use_simple_bp)
+	if (use_bp)
 	{
 		if (nd_eval != NULL) nd_eval->AddCycle();
 		for (i32s n1 = 0;n1 < GetSetup()->GetMMAtomCount();n1++)
 		{
-			f64 radius = bp_radius_solute;
+			f64 radius = bp_rad_solute;
 			f64 fc = bp_fc_solute;
 			
 			if (atmtab[n1]->flags & ATOMFLAG_IS_SOLVENT_ATOM)
 			{
-				radius = bp_radius_solvent;
+				radius = bp_rad_solvent;
 				fc = bp_fc_solvent;
 			}
 			
 			f64 t1a[3]; f64 t1b = 0.0;
 			for (i32s n2 = 0;n2 < 3;n2++)
 			{
-				f64 t9a = bp_center[n2];
+				f64 t9a = 0.0;//bp_center[n2];
 				f64 t9b = crd[l2g_mm[n1] * 3 + n2];
 				
 				t1a[n2] = t9a - t9b;
@@ -1039,17 +1139,20 @@ else E_solusolv += tote;
 
 /*################################################################################################*/
 
-eng1_mm_tripos52_nbt_mim::eng1_mm_tripos52_nbt_mim(setup * p1, i32u p2) : engine(p1, p2), eng1_mm(p1, p2), engine_pbc(p1, p2)
+eng1_mm_tripos52_nbt_mim::eng1_mm_tripos52_nbt_mim(setup * p1, i32u p2) :
+	engine(p1, p2),
+	eng1_mm(p1, p2),
+	engine_pbc(p1, p2)
 {
-	fGL mindim = GetSetup()->GetModel()->periodic_box_HALFdim[0];
-	if (GetSetup()->GetModel()->periodic_box_HALFdim[1] < mindim) mindim = GetSetup()->GetModel()->periodic_box_HALFdim[1];
-	if (GetSetup()->GetModel()->periodic_box_HALFdim[2] < mindim) mindim = GetSetup()->GetModel()->periodic_box_HALFdim[2];
+	fGL mindim = box_HALFdim[0];
+	if (box_HALFdim[1] < mindim) mindim = box_HALFdim[1];
+	if (box_HALFdim[2] < mindim) mindim = box_HALFdim[2];
 	
 	sw1 = 0.6; if (sw1 < (mindim - 0.4)) sw1 = mindim - 0.4;	// will trigger if boxdim < 2.0 nm!!!
 	sw2 = shft1 = mindim - 0.2;
 	
 	limit = mindim;
-	update = true;
+	update_neighbor_list = true;
 	
 	// calculate the actual values...
 	
@@ -1064,6 +1167,21 @@ eng1_mm_tripos52_nbt_mim::eng1_mm_tripos52_nbt_mim(setup * p1, i32u p2) : engine
 	limit = limit * limit;
 	
 	nbt1_vector.reserve(250000);
+	
+	// check from the distance constraints if there are any skip requests.
+	
+	int nskip = 0;
+	for (i32u cind = 0;cind < c_dst_vector.size();cind++)
+	{
+		if (!c_dst_vector[cind].skipNB) continue;
+		nskip++;
+	}
+	
+	if (nskip > 0)
+	{
+		static const char msg[] = "Cannot skip the nonbonded terms\nas requested in distance constraints.";
+		GetSetup()->GetModel()->WarningMessage(msg);
+	}
 }
 
 eng1_mm_tripos52_nbt_mim::~eng1_mm_tripos52_nbt_mim(void)
@@ -1079,7 +1197,7 @@ void eng1_mm_tripos52_nbt_mim::ComputeNBT1(i32u p1)
 	
 	atom ** atmtab = GetSetup()->GetMMAtoms();
 	
-	if (update) UpdateTerms();
+	if (update_neighbor_list) UpdateTerms();
 	
 	// the nonbonded terms begin...
 	// the nonbonded terms begin...
@@ -1101,13 +1219,13 @@ void eng1_mm_tripos52_nbt_mim::ComputeNBT1(i32u p1)
 			
 			t1a[n2] = t2a - t2b;
 			
-			if (t1a[n2] < -GetSetup()->GetModel()->periodic_box_HALFdim[n2])
+			if (t1a[n2] < -box_HALFdim[n2])
 			{
-				t1a[n2] += 2.0 * GetSetup()->GetModel()->periodic_box_HALFdim[n2];
+				t1a[n2] += 2.0 * box_HALFdim[n2];
 			}
-			else if (t1a[n2] > +GetSetup()->GetModel()->periodic_box_HALFdim[n2])
+			else if (t1a[n2] > +box_HALFdim[n2])
 			{
-				t1a[n2] -= 2.0 * GetSetup()->GetModel()->periodic_box_HALFdim[n2];
+				t1a[n2] -= 2.0 * box_HALFdim[n2];
 			}
 			
 			t1b += t1a[n2] * t1a[n2];
@@ -1214,6 +1332,8 @@ else E_solusolv += tote;
 
 void eng1_mm_tripos52_nbt_mim::UpdateTerms(void)
 {
+	update_neighbor_list = false;	// processed...
+	
 	model * mdl = GetSetup()->GetModel();
 	atom ** atmtab = GetSetup()->GetMMAtoms();
 //	bond ** bndtab = GetSetup()->GetMMBonds();
@@ -1221,7 +1341,6 @@ void eng1_mm_tripos52_nbt_mim::UpdateTerms(void)
 //	ostream * ostr = tripos52_tables::GetInstance()->ostr;
 	ostream * ostr = & cout;
 	
-	update = false;
 	nbt1_vector.resize(0);
 	
 	if (ostr != NULL) (* ostr) << "creating nbt1-terms: ";
@@ -1248,13 +1367,13 @@ void eng1_mm_tripos52_nbt_mim::UpdateTerms(void)
 					
 					t1a = t2a - t2b;
 					
-					if (t1a < -GetSetup()->GetModel()->periodic_box_HALFdim[n1])
+					if (t1a < -box_HALFdim[n1])
 					{
-						t1a += 2.0 * GetSetup()->GetModel()->periodic_box_HALFdim[n1];
+						t1a += 2.0 * box_HALFdim[n1];
 					}
-					else if (t1a > +GetSetup()->GetModel()->periodic_box_HALFdim[n1])
+					else if (t1a > +box_HALFdim[n1])
 					{
-						t1a -= 2.0 * GetSetup()->GetModel()->periodic_box_HALFdim[n1];
+						t1a -= 2.0 * box_HALFdim[n1];
 					}
 					
 					t1b += t1a * t1a;
@@ -1271,7 +1390,7 @@ void eng1_mm_tripos52_nbt_mim::UpdateTerms(void)
 				
 				bool is14 = (test != range_cr2[ind1 + 1]);	// if this is true, the atoms are 1-4 related.
 				
-				// see also eng1_mm_tripos52_nbt_mbp ctor ; should be the same!!!
+				// see also eng1_mm_tripos52_nbt_bp ctor ; should be the same!!!
 				// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 				
 				mm_tripos52_nbt1 newnbt1;
@@ -1289,28 +1408,34 @@ void eng1_mm_tripos52_nbt_mim::UpdateTerms(void)
 	i32s total_err = nbt1_err;
 	if (total_err)
 	{
-		char mbuff1[256];
-		ostrstream str1(mbuff1, sizeof(mbuff1));
+		ostringstream str1;
 		str1 << "WARNING : there were " << total_err << " missing parameters in the nonbonded terms." << endl << ends;
-		mdl->PrintToLog(mbuff1);
+		mdl->PrintToLog(str1.str().c_str());
 	}
 }
 
 /*################################################################################################*/
 
-eng1_mm_tripos52_mbp::eng1_mm_tripos52_mbp(setup * p1, i32u p2) : engine(p1, p2), eng1_mm(p1, p2),
-	engine_mbp(p1, p2), eng1_mm_tripos52_bt(p1, p2), eng1_mm_tripos52_nbt_mbp(p1, p2)
+eng1_mm_tripos52_bp::eng1_mm_tripos52_bp(setup * p1, i32u p2) :
+	engine(p1, p2),
+	eng1_mm(p1, p2),
+	engine_bp(p1, p2),
+	eng1_mm_tripos52_bt(p1, p2),
+	eng1_mm_tripos52_nbt_bp(p1, p2)
 {
 }
 
-eng1_mm_tripos52_mbp::~eng1_mm_tripos52_mbp(void)
+eng1_mm_tripos52_bp::~eng1_mm_tripos52_bp(void)
 {
 }
 
 /*################################################################################################*/
 
-eng1_mm_tripos52_mim::eng1_mm_tripos52_mim(setup * p1, i32u p2) : engine(p1, p2), eng1_mm(p1, p2),
-	eng1_mm_tripos52_bt(p1, p2), eng1_mm_tripos52_nbt_mim(p1, p2)
+eng1_mm_tripos52_mim::eng1_mm_tripos52_mim(setup * p1, i32u p2) :
+	engine(p1, p2),
+	eng1_mm(p1, p2),
+	eng1_mm_tripos52_bt(p1, p2),
+	eng1_mm_tripos52_nbt_mim(p1, p2)
 {
 }
 
