@@ -19,10 +19,10 @@ struct mm_tripos52_bt3;		// torsion
 struct mm_tripos52_nbt1;	// nonbonded
 
 class eng1_mm_tripos52_bt;		// bonded part
-class eng1_mm_tripos52_nbt_mbp;		// nonbonded part; (optional) modified boundary potential
+class eng1_mm_tripos52_nbt_bp;		// nonbonded part; (optional) boundary potential
 class eng1_mm_tripos52_nbt_mim;		// nonbonded part; minimum image model
 
-class eng1_mm_tripos52_mbp;
+class eng1_mm_tripos52_bp;
 class eng1_mm_tripos52_mim;
 
 /*################################################################################################*/
@@ -168,7 +168,7 @@ class eng1_mm_tripos52_bt : virtual public eng1_mm
 
 /// molecular mechanics; nonbonded part, non-periodic.
 
-class eng1_mm_tripos52_nbt_mbp : virtual public eng1_mm, virtual public engine_mbp
+class eng1_mm_tripos52_nbt_bp : virtual public eng1_mm, virtual public engine_bp
 {
 	protected:
 	
@@ -176,8 +176,8 @@ class eng1_mm_tripos52_nbt_mbp : virtual public eng1_mm, virtual public engine_m
 	
 	public:
 	
-	eng1_mm_tripos52_nbt_mbp(setup *, i32u);
-	virtual ~eng1_mm_tripos52_nbt_mbp(void);
+	eng1_mm_tripos52_nbt_bp(setup *, i32u);
+	virtual ~eng1_mm_tripos52_nbt_bp(void);
 	
 	protected:
 	
@@ -222,12 +222,12 @@ class eng1_mm_tripos52_nbt_mim : virtual public eng1_mm, public engine_pbc
 
 /*################################################################################################*/
 
-class eng1_mm_tripos52_mbp : public eng1_mm_tripos52_bt, public eng1_mm_tripos52_nbt_mbp
+class eng1_mm_tripos52_bp : public eng1_mm_tripos52_bt, public eng1_mm_tripos52_nbt_bp
 {
 	public:
 	
-	eng1_mm_tripos52_mbp(setup *, i32u);
-	~eng1_mm_tripos52_mbp(void);
+	eng1_mm_tripos52_bp(setup *, i32u);
+	~eng1_mm_tripos52_bp(void);
 };
 
 class eng1_mm_tripos52_mim : public eng1_mm_tripos52_bt, public eng1_mm_tripos52_nbt_mim
