@@ -36,22 +36,22 @@ class eng1_qm_mopac : public eng1_qm
 	~eng1_qm_mopac(void);
 	
 	static eng1_qm_mopac * GetLock(void) { return mopac_lock; }
-
-	virtual i32s GetOrbitalCount(void);	// virtual
-	virtual f64 GetOrbitalEnergy(i32s);	// virtual
 	
-	virtual i32s GetElectronCount(void);	// virtual
-
-	virtual void Compute(i32u);	// virtual
-
-	virtual void SetupPlotting(void);	// virtual
+	i32s GetOrbitalCount(void);		// virtual
+	f64 GetOrbitalEnergy(i32s);		// virtual
 	
-	virtual fGL GetESP(fGL *, fGL *);	// virtual
+	i32s GetElectronCount(void);		// virtual
 	
-	virtual fGL GetElDens(fGL *, fGL *);	// virtual
+	void Compute(i32u, bool = false);	// virtual
 	
-	virtual fGL GetOrbital(fGL *, fGL *);	// virtual
-	virtual fGL GetOrbDens(fGL *, fGL *);	// virtual
+	void SetupPlotting(void);		// virtual
+	
+	fGL GetESP(fGL *, fGL *);		// virtual
+	
+	fGL GetElDens(fGL *, fGL *);		// virtual
+	
+	fGL GetOrbital(fGL *, fGL *);		// virtual
+	fGL GetOrbDens(fGL *, fGL *);		// virtual
 };
 
 /*################################################################################################*/
