@@ -85,7 +85,7 @@ class eng1_qm_mpqc : public eng1_qm
 	Ref<Wavefunction> wfn;			// for plotting...
 	
 #endif
-
+	
 	public:
 	
 	eng1_qm_mpqc(setup *, i32u, i32u);
@@ -94,25 +94,25 @@ class eng1_qm_mpqc : public eng1_qm
 	void GetInputFileName(char *, i32u);
 	void WriteInputFile(ofstream &, i32s);
 	
-	virtual i32s GetOrbitalCount(void);	// virtual
-	virtual f64 GetOrbitalEnergy(i32s);	// virtual
+	i32s GetOrbitalCount(void);		// virtual
+	f64 GetOrbitalEnergy(i32s);		// virtual
 	
-	virtual i32s GetElectronCount(void);	// virtual
+	i32s GetElectronCount(void);			// virtual
 	
 /**	It seems that we get the energy in atomic units... A set of unit conversion 
 	functions should be made that makes conversions si-units <-> atomic units <-> 
 	whatever units. These engine classes should have some "states" which determine 
 	units that are printed out ?!?!?!?!?!?
 */
-	virtual void Compute(i32u);		// virtual
+	void Compute(i32u, bool = false);	// virtual
 	
-	virtual void SetupPlotting(void);	// virtual
+	void SetupPlotting(void);		// virtual
 	
-	virtual fGL GetESP(fGL *, fGL *);	// virtual
-	virtual fGL GetElDens(fGL *, fGL *);	// virtual
+	fGL GetESP(fGL *, fGL *);		// virtual
+	fGL GetElDens(fGL *, fGL *);		// virtual
 	
-	virtual fGL GetOrbital(fGL *, fGL *);	// virtual
-	virtual fGL GetOrbDens(fGL *, fGL *);	// virtual
+	fGL GetOrbital(fGL *, fGL *);		// virtual
+	fGL GetOrbDens(fGL *, fGL *);		// virtual
 };
 
 /*################################################################################################*/
