@@ -48,6 +48,7 @@ struct sf_nbt3_arc;
 #define LAYER_LIMIT	0.10	// THEREFORE DISABLED IN CODE!!! but still makes the arrays -> memory!!!
 
 static const int size_nl[LAYERS] = { 100, 200, 400 };
+#define MAX_SIZE_NL 400		// size_nl[2] = 400
 
 #define SIZE_NT		100
 #define SIZE_COI	100
@@ -119,17 +120,16 @@ void eng1_sf_param_MakeCopy(eng1_sf_param &, eng1_sf_param &);
 
 class sf_chn		// chain
 {
-	protected:
+//	protected:	// strict...
+	public:		// loose ; this is for ghemical...
 	
 	vector<sf_res> res_vector;
 	
-	friend class setup1_sf;
 	friend void CopyCRD(model *, engine *, i32u);
 	friend void CopyCRD(engine *, model *, i32u);
-	friend class eng1_sf;
 	
-	friend class project;		// FIXME : for ghemical; read access to res_vector???
-	friend class ribbon;		// FIXME : for ghemical; read access to res_vector???
+	friend class setup1_sf;
+	friend class eng1_sf;
 	
 	public:
 	
@@ -157,9 +157,10 @@ class sf_res		// residue
 	
 	i32u state;
 	
-	friend class setup1_sf;
 	friend void CopyCRD(model *, engine *, i32u);
 	friend void CopyCRD(engine *, model *, i32u);
+	
+	friend class setup1_sf;
 	friend class eng1_sf;
 	
 	public:
@@ -183,16 +184,36 @@ class sf_dsb		// disulphide bridge
 	
 	i32s chn[2];
 	i32s res[2];
-
+	
 	friend class setup1_sf;
 	friend class eng1_sf;
 	
-	friend class project;		// FIXME : for ghemical
-	
 	public:
 	
 	sf_dsb(void) { }
 	~sf_dsb(void) { }
+	
+	i32s GetChn(i32s i)
+	{
+		if (i < 0 || i > 1)
+		{
+			cout << "sf_dsb::GetChn() : index overflow!" << endl;
+			exit(EXIT_FAILURE);
+		}
+		
+		return chn[i];
+	}
+	
+	i32s GetRes(i32s i)
+	{
+		if (i < 0 || i > 1)
+		{
+			cout << "sf_dsb::GetRes() : index overflow!" << endl;
+			exit(EXIT_FAILURE);
+		}
+		
+		return res[i];
+	}
 };
 
 /*################################################################################################*/
@@ -208,7 +229,6 @@ class sf_helix4info
 	vector<atom *> cg_donacc;
 	
 	friend class setup1_sf;
-	friend class project;		// FIXME : for ghemical
 	
 	public:
 	
@@ -226,6 +246,16 @@ class sf_helix4info
 	i32u GetChn(void) { return chn; }
 	i32u GetResBgn(void) { return res[0]; }
 	i32u GetResEnd(void) { return res[1]; }
+	
+	void CopyDAVectUA(vector<atom *> & v)
+	{
+		v = ua_donacc;
+	}
+	
+	void CopyDAVectCG(vector<atom *> & v)
+	{
+		v = cg_donacc;
+	}
 };
 
 class sf_strandinfo
@@ -243,9 +273,9 @@ class sf_strandinfo
 		res[0] = rb;
 		res[1] = re;
 		
-		cout << "DEBUG sf_strandinfo ; created record ";
-		cout << "c= " << chn << " rb= " << res[0] << " re= " << res[1];
-		cout << endl;
+	//	cout << "DEBUG sf_strandinfo ; created record ";
+	//	cout << "c= " << chn << " rb= " << res[0] << " re= " << res[1];
+	//	cout << endl;
 	}
 	
 	~sf_strandinfo(void)
@@ -268,7 +298,6 @@ class sf_strandpair
 	bool parallel;
 	
 	friend class setup1_sf;
-	friend class project;		// FIXME : for ghemical
 	
 	public:
 	
@@ -322,31 +351,29 @@ class sf_strandpair
 
 class setup1_sf : virtual public setup
 {
-//	protected:
-	public:
-	
-	eng1_sf_param prm;
-	
-	protected:
-	
-	bool united_atoms;
-	bool exp_solv;
+//	protected:	// strict...
+	public:		// loose ; this is for ghemical...
 	
 	vector<sf_chn> chn_vector;
 	vector<sf_dsb> dsb_vector;
 	
+	bool united_atoms;
+	
 	vector<sf_helix4info> hi_vector;
 	vector<sf_strandinfo> si_vector;
 	vector<sf_strandpair> sp_vector;
 	
+	vector<atom *> tb_vis1_vector;
+	vector<float> tb_vis2_vector;
+	
+	protected:
+	
+	eng1_sf_param prm;
+	
 	friend void CopyCRD(model *, engine *, i32u);
 	friend void CopyCRD(engine *, model *, i32u);
-	friend class eng1_sf;
 	
-	friend class setup_druid;	// FIXME:  for ghemical		// for BBB
-	friend class setup_dialog;	// FIXME:  for ghemical		// for HEAD
-	friend class project;		// FIXME : for ghemical; read access to chn_vector???
-	friend class ribbon;		// FIXME : for ghemical; read access to chn_vector???
+	friend class eng1_sf;
 	
 	public:
 	
@@ -619,10 +646,9 @@ struct sf_nbt3_arc		///< SASA positively oriented arc.
 
 /*################################################################################################*/
 
-class eng1_sf : virtual public engine_mbp
+class eng1_sf : virtual public engine_bp
 {
-//	protected:
-	public:		// FIX_ME
+	protected:
 	
 	i32u * l2g_sf;		///< The local-to-global lookup table.
 	i32s * index_chn;	///< This maps the (local) atoms to setup1_sf chains/residues.
@@ -683,6 +709,11 @@ class eng1_sf : virtual public engine_mbp
 	
 	f64 constraints;
 	
+	friend void CopyCRD(model *, engine *, i32u);
+	friend void CopyCRD(engine *, model *, i32u);
+	
+	friend class moldyn_langevin;
+	
 	public:
 	
 	eng1_sf(setup *, i32u, bool, bool);
@@ -691,7 +722,7 @@ class eng1_sf : virtual public engine_mbp
 	void SetupCharges(void);
 	void GetChgGrpVar(i32s, i32s &, i32s &);
 	
-	virtual void Compute(i32u);	// virtual
+	void Compute(i32u, bool = false);	// virtual
 	
 	i32s FindTorsion(atom *, atom *, atom *, atom *) { return NOT_DEFINED; }	// virtual
 	bool SetTorsionConstraint(i32s, f64, f64, bool) { return false; }		// virtual
