@@ -60,7 +60,7 @@ class setup2_qm_mm : public setup1_qm, public setup1_mm
 
 #ifdef ENABLE_MOPAC7
 
-class eng2_qm_mm_mopac : public eng1_qm_mopac, public eng1_mm_default_mbp
+class eng2_qm_mm_mopac : public eng1_qm_mopac, public eng1_mm_default_bp
 {
 	protected:
 	
@@ -74,7 +74,7 @@ class eng2_qm_mm_mopac : public eng1_qm_mopac, public eng1_mm_default_mbp
 	
 	i32s GetElectronCount(void);	// virtual
 	
-	void Compute(i32u);	// virtual
+	void Compute(i32u, bool = false);	// virtual
 	
 	i32s FindTorsion(atom *, atom *, atom *, atom *) { cout << "BUG: FindTorsion() is not yet implemented at eng2_qm_mm!" << endl; exit(EXIT_FAILURE); }
 	bool SetTorsionConstraint(i32s, f64, f64, bool) { cout << "BUG: SetTorsionConstraint() is not yet implemented at eng2_qm_mm!" << endl; exit(EXIT_FAILURE); }
@@ -97,7 +97,7 @@ class eng2_qm_mm_mopac : public eng1_qm_mopac, public eng1_mm_default_mbp
 
 #ifdef ENABLE_MPQC
 
-class eng2_qm_mm_mpqc : public eng1_qm_mpqc, public eng1_mm_default_mbp
+class eng2_qm_mm_mpqc : public eng1_qm_mpqc, public eng1_mm_default_bp
 {
 	protected:
 	
@@ -111,7 +111,7 @@ class eng2_qm_mm_mpqc : public eng1_qm_mpqc, public eng1_mm_default_mbp
 	
 	i32s GetElectronCount(void);	// virtual
 	
-	void Compute(i32u);	// virtual
+	void Compute(i32u, bool = false);	// virtual
 	
 	i32s FindTorsion(atom *, atom *, atom *, atom *) { cout << "BUG: FindTorsion() is not yet implemented at eng2_qm_mm!" << endl; exit(EXIT_FAILURE); }
 	bool SetTorsionConstraint(i32s, f64, f64, bool) { cout << "BUG: SetTorsionConstraint() is not yet implemented at eng2_qm_mm!" << endl; exit(EXIT_FAILURE); }
