@@ -9,6 +9,8 @@
 
 #include "engine.h"
 
+#include "v3d.h"
+
 #include "model.h"
 
 /*################################################################################################*/
@@ -337,6 +339,10 @@ engine::engine(setup * p1, i32u p2)
 	
 	if (p2 > 1) d2 = new f64[natm * natm * 9];
 	else d2 = NULL;
+	
+	virial[0] = 0.0;
+	virial[1] = 0.0;
+	virial[2] = 0.0;
 }
 
 engine::~engine(void)
@@ -389,6 +395,19 @@ f64 engine::GetGradientVectorLength(void)
 	return sqrt(sum);
 }
 
+void engine::ScaleCRD(f64 kx, f64 ky, f64 kz)
+{
+	atom ** glob_atmtab = GetSetup()->GetAtoms();
+	for (i32s n1 = 0;n1 < natm;n1++)
+	{
+		if (glob_atmtab[n1]->flags & ATOMFLAG_USER_LOCKED) continue;
+		
+		crd[n1 * 3 + 0] *= kx;
+		crd[n1 * 3 + 1] *= ky;
+		crd[n1 * 3 + 2] *= kz;
+	}
+}
+
 void engine::DoSHAKE(void)
 {
 }
@@ -558,21 +577,17 @@ void CopyCRD(engine * p1, model * p2, i32u p3)
 
 /*################################################################################################*/
 
-engine_mbp::engine_mbp(setup * p1, i32u p2) : engine(p1, p2)
+engine_bp::engine_bp(setup * p1, i32u p2) : engine(p1, p2)
 {
-	use_simple_bp = GetSetup()->GetModel()->use_boundary_potential;
-	bp_radius_solute = GetSetup()->GetModel()->boundary_potential_radius1;
-	bp_radius_solvent = GetSetup()->GetModel()->boundary_potential_radius2;
-	
-	bp_center[0] = 0.0;
-	bp_center[1] = 0.0;
-	bp_center[2] = 0.0;
+	use_bp = GetSetup()->GetModel()->use_boundary_potential;
+	bp_rad_solute = GetSetup()->GetModel()->saved_boundary_potential_rad_solute;
+	bp_rad_solvent = GetSetup()->GetModel()->saved_boundary_potential_rad_solvent;
 	
 	nd_eval = NULL;
 	rdf_eval = NULL;
 }
 
-engine_mbp::~engine_mbp(void)
+engine_bp::~engine_bp(void)
 {
 	if (nd_eval != NULL) delete nd_eval;
 	if (rdf_eval != NULL) delete rdf_eval;
@@ -582,34 +597,53 @@ engine_mbp::~engine_mbp(void)
 
 engine_pbc::engine_pbc(setup * p1, i32u p2) : engine(p1, p2)
 {
-	// assume that GetModel()->GatherGroups() is done...
+	f64 * tmp = GetSetup()->GetModel()->saved_periodic_box_HALFdim;
 	
-	nmol_mm = 0;
+	box_HALFdim[0] = tmp[0];
+	box_HALFdim[1] = tmp[1];
+	box_HALFdim[2] = tmp[2];
 	
-// count the molecules present in the MM part.
-// since the "mol" level is the highest criteria in sorting, atoms in a molecule should
+	tmp = NULL;
+	
+	num_mol = 0;
+	
+// count the molecules present in the full atom set ; since the "mol"
+// level is the highest criteria in sorting, atoms in a molecule should
 // be adjacent -> a continuous range of pointers.
-
+	
+	if (!GetSetup()->GetModel()->IsGroupsSorted())
+	{
+		cout << "engine_pbc ctor ; IsGroupsSorted() returned false!" << endl;
+		exit(EXIT_FAILURE);
+	}
+	
 // here we calculate the molecule locations precisely, but a simple trigger atom could be used as well...
-
+	
 	i32s previous = -123;	// what is the safest setting here??? NOT_DEFINED might be used there???
-	atom ** atmtab = GetSetup()->GetMMAtoms();
-	for (i32s index = 0;index < GetSetup()->GetMMAtomCount();index++)
+	
+	atom ** atmtab = GetSetup()->GetAtoms();
+	for (i32s index = 0;index < GetSetup()->GetAtomCount();index++)
 	{
 		if (atmtab[index]->id[0] != previous)
 		{
-			nmol_mm++;
+			num_mol++;
 			previous = atmtab[index]->id[0];
 		}
 	}
 	
-	mrange = new i32s[nmol_mm + 1];
+	mrange = new i32s[num_mol + 1];
 	
-	mrange[0] = 0; i32s index = 1;
-	for (i32s n1 = 0;n1 < nmol_mm;n1++)
+	mrange[0] = 0; i32s a_index = 0;	// a_index counts LOCAL atom indices.
+	for (i32s n1 = 0;n1 < num_mol;n1++)
 	{
-		while (index < GetSetup()->GetMMAtomCount() && atmtab[index]->id[0] == n1) index++;
-		mrange[n1 + 1] = index;
+		i32s m_index = atmtab[a_index]->id[0];
+		
+		// m_index counts atom::id[0] molecule numbers.
+		// m_index MAY APPEAR DISCONTINUOUS IF eng1_sf!
+		// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+		
+		while (a_index < GetSetup()->GetAtomCount() && atmtab[a_index]->id[0] == m_index) a_index++;
+		mrange[n1 + 1] = a_index;
 	}
 }
 
@@ -620,8 +654,8 @@ engine_pbc::~engine_pbc(void)
 
 void engine_pbc::CheckLocations(void)
 {
-	atom ** atmtab = GetSetup()->GetMMAtoms();
-	for (i32s n1 = 0;n1 < nmol_mm;n1++)
+	atom ** atmtab = GetSetup()->GetAtoms();
+	for (i32s n1 = 0;n1 < num_mol;n1++)
 	{
 		f64 sum[3] = { 0.0, 0.0, 0.0 };
 		f64 ac = (f64) (mrange[n1 + 1] - mrange[n1]);
@@ -638,20 +672,20 @@ void engine_pbc::CheckLocations(void)
 		{
 			f64 test = sum[n2] / ac;
 			
-			if (test < -GetSetup()->GetModel()->periodic_box_HALFdim[n2])
+			if (test < -box_HALFdim[n2])
 			{
 				for (i32s n3 = mrange[n1];n3 < mrange[n1 + 1];n3++)
 				{
 					i32u index = atmtab[n3]->varind;
-					crd[index * 3 + n2] += 2.0 * GetSetup()->GetModel()->periodic_box_HALFdim[n2];
+					crd[index * 3 + n2] += 2.0 * box_HALFdim[n2];
 				}
 			}
-			else if (test > +GetSetup()->GetModel()->periodic_box_HALFdim[n2])
+			else if (test > +box_HALFdim[n2])
 			{
 				for (i32s n3 = mrange[n1];n3 < mrange[n1 + 1];n3++)
 				{
 					i32u index = atmtab[n3]->varind;
-					crd[index * 3 + n2] -= 2.0 * GetSetup()->GetModel()->periodic_box_HALFdim[n2];
+					crd[index * 3 + n2] -= 2.0 * box_HALFdim[n2];
 				}
 			}
 		}
@@ -664,15 +698,15 @@ void engine_pbc::CheckLocations(void)
 
 /*################################################################################################*/
 
-number_density_evaluator::number_density_evaluator(engine_mbp * p1, bool p2, i32s p3)
+number_density_evaluator::number_density_evaluator(engine_bp * p1, bool p2, i32s p3)
 {
 	eng = p1;
 	linear = p2;
 	classes = p3;
 	
-	if (!eng->use_simple_bp)
+	if (!eng->use_bp)
 	{
-		cout << "number_density_evaluator : eng->use_simple_bp was false!" << endl;
+		cout << "number_density_evaluator : eng->use_bp was false!" << endl;
 		exit(EXIT_FAILURE);
 	}
 	
@@ -699,7 +733,7 @@ void number_density_evaluator::UpdateClassLimits(void)
 		f64 prev_radius = 0.0;
 		for (i32s n1 = 0;n1 < classes;n1++)
 		{
-			f64 next_radius = eng->bp_radius_solvent * ((f64) (n1 + 1)) / (f64) classes;
+			f64 next_radius = eng->bp_rad_solvent * ((f64) (n1 + 1)) / (f64) classes;
 			upper_limits[n1] = next_radius;
 			
 			f64 volume1 = 4.0 * M_PI * prev_radius * prev_radius * prev_radius / 3.0;
@@ -712,7 +746,7 @@ void number_density_evaluator::UpdateClassLimits(void)
 	}
 	else
 	{
-		f64 volume1 = 4.0 * M_PI * eng->bp_radius_solvent * eng->bp_radius_solvent * eng->bp_radius_solvent / 3.0;
+		f64 volume1 = 4.0 * M_PI * eng->bp_rad_solvent * eng->bp_rad_solvent * eng->bp_rad_solvent / 3.0;
 		f64 volume2 = volume1 / (f64) classes;
 		
 		f64 prev_radius = 0.0;
@@ -759,7 +793,7 @@ void number_density_evaluator::PrintResults(ostream & str)
 
 /*################################################################################################*/
 
-radial_density_function_evaluator::radial_density_function_evaluator(engine_mbp * p1, i32s p2, f64 gb, f64 ge, f64 cb, f64 ce)
+radial_density_function_evaluator::radial_density_function_evaluator(engine_bp * p1, i32s p2, f64 gb, f64 ge, f64 cb, f64 ce)
 {
 	eng = p1;
 	classes = p2;
@@ -772,9 +806,9 @@ radial_density_function_evaluator::radial_density_function_evaluator(engine_mbp
 	
 	if (count_begin < 0.0)
 	{
-		if (!eng->use_simple_bp)
+		if (!eng->use_bp)
 		{
-			cout << "radial_density_function_evaluator : eng->use_simple_bp was false!" << endl;
+			cout << "radial_density_function_evaluator : eng->use_bp was false!" << endl;
 			exit(EXIT_FAILURE);
 		}
 		
