@@ -10,12 +10,13 @@
 #ifndef ENGINE_H
 #define ENGINE_H
 
+#include "libghemicaldefine.h"
 //#include "libghemicalconfig2.h"
 
 class setup;
 class engine;
 
-class engine_mbp;	// modified boundary potential
+class engine_bp;	// boundary potential
 class engine_pbc;	// periodic boundary conditions
 
 class number_density_evaluator;
@@ -69,13 +70,6 @@ class setup
 	
 	atom ** sf_atmtab; i32s sf_natm;
 	
-	friend class model;				// needs to access current_eng_index.
-	friend class transition_state_search;		// needs to access current_eng.
-	
-	friend class project;		// FOR GHEMICAL!!!
-	friend class setup_druid;	// FOR GHEMICAL!!!	// needs to access current_eng_index.	// for BBB
-	friend class setup_dialog;	// FOR GHEMICAL!!!	// needs to access current_eng_index.	// for HEAD
-	
 	public:
 	
 	setup(model *);
@@ -111,7 +105,9 @@ class setup
 	i32s GetSFAtomCount(void) { return sf_natm; }
 	
 	engine * GetCurrentEngine(void) { return current_eng; }
+	
 	i32s GetCurrEngIndex(void) { return current_eng_index; }
+	void SetCurrEngIndex(i32s index) { current_eng_index = index; }		// only file operations etc should use this...
 	
 	virtual void UpdateAtomFlags(void) = 0;
 	
@@ -126,8 +122,6 @@ class setup
 	virtual const char * GetEngineName(i32u) = 0;
 	virtual const char * GetClassName(void) = 0;
 	
-	void SetCurrentEngIndex(i32s index) { current_eng_index = index; }	// only file operations etc should use this...
-	
 	void CreateCurrentEngine(void);				///< Create the current_eng object for plotting...
 	void DiscardCurrentEngine(void);
 	
@@ -156,6 +150,11 @@ Since there is, for example, only one table for coordinates and derivatives at t
 the different derived engine classes must create a suitable lookup table that maps the local tables 
 back to the global one. */
 
+// here are some common error/warning messages:
+// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+
+#define CONSTRAINTS_NOT_SUPPORTED "Sorry ; constraints are not yet supported by this engine class."
+
 class engine
 {
 	private:
@@ -163,12 +162,12 @@ class engine
 	setup * stp;
 	
 	protected:
-
+	
 	i32s natm;
 	
 	f64 * crd;
-
-	f64 energy;	// GetEnergy() ???
+	
+	f64 energy;
 	
 // add atom::ecomp_grp_ind ; define/register the new groups in the model object (and save them in files).
 // ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
@@ -183,24 +182,26 @@ class engine
 	f64 * d1;	// GetD1() ???
 	f64 * d2;	// GetD2() ???
 	
-	friend class model;
-	friend class project;
+	f64 virial[3];
 	
-	friend class geomopt;				// NOT NECESSARY
-	friend class moldyn;				// NOT NECESSARY
-	friend class moldyn_langevin;			// NOT NECESSARY
-	
-	friend class random_search;			// NOT NECESSARY
-	friend class systematic_search;			// NOT NECESSARY
-	friend class monte_carlo_search;		// NOT NECESSARY
-	friend class transition_state_search;		// needs to access crd.
-	friend class stationary_state_search;		// needs to access d1.
-	
-	public:
+	bool update_neighbor_list;
 	
 	friend void CopyCRD(model *, engine *, i32u);
 	friend void CopyCRD(engine *, model *, i32u);
 	
+	friend class model;
+	
+	friend class geomopt;
+	
+	friend class moldyn;
+	friend class moldyn_langevin;
+	
+	friend class random_search;
+	friend class systematic_search;
+	friend class monte_carlo_search;
+	friend class transition_state_search;
+	friend class stationary_state_search;
+	
 	public:
 	
 	engine(setup *, i32u);
@@ -212,7 +213,14 @@ class engine
 	void Check(i32u);			///< This is for debugging; will compare the computed gradient to a numerical one.
 	f64 GetGradientVectorLength(void);	///< This calculates the gradient vector length (using the d1 array).
 	
-	virtual void Compute(i32u) = 0;		///< Will compute the energy, and the gradient if needed.
+	void ScaleCRD(f64, f64, f64);
+	
+/**	This will request an update for the NB-term list (neighbor list) at the next
+	energy evaluation.
+*/
+	void RequestNeighborListUpdate(void) { update_neighbor_list = true; }
+	
+	virtual void Compute(i32u, bool = false) = 0;	///< Will compute the energy, and the gradient if needed.
 	
 	virtual void DoSHAKE(void);		///< This is an optional method that contains some SHAKE-like corrections to atomic coordinates ; is called before calculating forces in MD.
 	
@@ -263,19 +271,19 @@ void CopyCRD(engine *, model *, i32u);
 
 /*################################################################################################*/
 
-/// A base engine class for systems that utiliza a (modified) boundary potential.
+/// A base engine class for systems that utilize a boundary potential.
 
-class engine_mbp : virtual public engine
+class engine_bp : virtual public engine
 {
 	protected:
 	
-	bool use_simple_bp; f64 bp_center[3];
-	f64 bp_radius_solute; f64 bp_fc_solute;
-	f64 bp_radius_solvent; f64 bp_fc_solvent;
+	bool use_bp;
+	f64 bp_rad_solute; f64 bp_fc_solute;
+	f64 bp_rad_solvent; f64 bp_fc_solvent;
 	
 	number_density_evaluator * nd_eval;
 	radial_density_function_evaluator * rdf_eval;
-
+	
 	friend class model;
 	
 	friend class number_density_evaluator;
@@ -283,8 +291,8 @@ class engine_mbp : virtual public engine
 	
 	public:
 	
-	engine_mbp(setup *, i32u);
-	virtual ~engine_mbp(void);
+	engine_bp(setup *, i32u);
+	virtual ~engine_bp(void);
 };
 
 /*################################################################################################*/
@@ -295,14 +303,15 @@ class engine_pbc : virtual public engine
 {
 	protected:
 	
-	i32s nmol_mm;
+	f64 box_HALFdim[3];
+	
+	i32s num_mol;
 	i32s * mrange;
 	
-	bool update;
+// TODO : how to use RDF-evaluator also here???
 	
-	// TODO : how to use RDF-evaluator also here???
-
 	friend class model;
+	friend class moldyn;
 	
 	public:
 	
@@ -324,7 +333,7 @@ class number_density_evaluator
 {
 	protected:
 	
-	engine_mbp * eng;
+	engine_bp * eng;
 	bool linear;
 	
 	i32s classes;
@@ -336,7 +345,7 @@ class number_density_evaluator
 	
 	public:
 	
-	number_density_evaluator(engine_mbp *, bool, i32s);
+	number_density_evaluator(engine_bp *, bool, i32s);
 	~number_density_evaluator(void);
 	
 	private:
@@ -374,7 +383,7 @@ class radial_density_function_evaluator
 {
 	protected:
 	
-	engine_mbp * eng;
+	engine_bp * eng;
 	
 	i32s classes;
 	f64 graph_begin;
@@ -388,13 +397,13 @@ class radial_density_function_evaluator
 	i32u cycles;
 	i32u * counts;
 	
-	friend class eng1_mm_tripos52_nbt_mbp;
-	friend class eng1_mm_default_nbt_mbp;
+	friend class eng1_mm_tripos52_nbt_bp;
+	friend class eng1_mm_default_nbt_bp;
 	friend class eng1_sf;
 	
 	public:
 	
-	radial_density_function_evaluator(engine_mbp *, i32s, f64, f64, f64 = -1.0, f64 = -1.0);
+	radial_density_function_evaluator(engine_bp *, i32s, f64, f64, f64 = -1.0, f64 = -1.0);
 	~radial_density_function_evaluator(void);
 	
 	private:
