@@ -31,9 +31,6 @@ class geomopt_param
 	
 	bool confirm;
 	
-	friend class model;
-	friend class geomopt_dialog;
-	
 	public:
 	
 	bool show_dialog;
@@ -52,6 +49,7 @@ class geomopt_param
 	geomopt_param(setup * su)
 	{
 		confirm = false;
+		
 		show_dialog = true;
 		
 		treshold_nsteps = 2500;
@@ -63,6 +61,9 @@ class geomopt_param
 	}
 	
 	~geomopt_param(void) { }
+	
+	bool GetConfirm(void) { return confirm; }
+	void Confirm(void) { confirm = true; }
 };
 
 /*################################################################################################*/
