@@ -44,6 +44,7 @@ class moldyn_param;		// moldyn.h
 
 #include "constraint.h"
 
+#include "chn_info.h"
 #include "seqbuild.h"
 
 #include <list>
@@ -140,7 +141,8 @@ class model
 	friend f64 HBondEnergy(model *, i32s *, i32s *);
 	
 	friend class sequencebuilder;
-
+	friend class mfinder;
+	
 	friend class engine;
 	friend class engine_bp;
 	friend class engine_pbc;
