@@ -17,9 +17,6 @@ class moldyn_param;
 class moldyn;
 class moldyn_langevin;
 
-class temperature_meter;
-class pressure_meter;
-
 /*################################################################################################*/
 
 #include "engine.h"
@@ -33,19 +30,28 @@ class moldyn_param
 	
 	bool confirm;
 	
-	friend class model;
-	friend class moldyn_dialog;
-	
 	public:
 	
 	bool show_dialog;
 	
-	i32s nsteps_h;		// heat
+	i32s nsteps_h;		// heating
 	i32s nsteps_e;		// equilibration
 	i32s nsteps_s;		// simulation
-	f64 temperature;
-	f64 timestep;
-	bool constant_e;
+	i32s nsteps_c;		// cooling
+	
+	f64 timestep;		// simulation timestep [fs]
+	
+	f64 target_T;		// target temperature [K]
+	f64 T_rtime_hc;		// temperature relaxation time [fs] heating/cooling
+	f64 T_rtime_es;		// temperature relaxation time [fs] equilibration/simulation
+	
+	f64 target_P;		// target pressure [bar]
+	f64 P_rtime;		// pressure relaxation time [fs]
+	f64 P_beta;		// isothermal compressibility [1/bar]
+	
+	bool constant_T;
+	bool constant_P;
+	
 	bool langevin;
 	
 	char filename[256];
@@ -55,23 +61,41 @@ class moldyn_param
 	moldyn_param(setup * su)
 	{
 		confirm = false;
+		
 		show_dialog = true;
 		
+	// set the defaults...
+	// ^^^^^^^^^^^^^^^^^^^
+		
 		nsteps_h = 5000;
 		nsteps_e = 5000;
-		nsteps_s = 10000;
-		temperature = 300.0;
+		nsteps_s = 18000;
+		nsteps_c = 2000;
+		
 		timestep = 0.5;
-		constant_e = false;
+	setup1_sf * susf = dynamic_cast<setup1_sf *>(su);
+	if (susf != NULL) timestep = 5.0;	// override...
+		
+		target_T = 300.0;
+		T_rtime_hc = 10.0 * timestep;
+		T_rtime_es = 100.0 * timestep;
+		
+		target_P = 1.000;
+		P_rtime = 100.0 * timestep;
+		P_beta = 4.57e-5;
+		
+		constant_T = true;
+		constant_P = false;
+		
 		langevin = false;
 		
 		strcpy(filename, "untitled.traj");
-		
-		setup1_sf * susf = dynamic_cast<setup1_sf *>(su);
-		if (susf != NULL) timestep = 5.0;			// override...
 	}
 	
 	~moldyn_param(void) { }
+	
+	bool GetConfirm(void) { return confirm; }
+	void Confirm(void) { confirm = true; }
 };
 
 /*################################################################################################*/
@@ -93,27 +117,37 @@ class moldyn
 	
 	engine * eng;
 	
-	f64 * vel;
-	f64 * acc;
+	f64 * vel;			// [1.0e+3 m/s]
+	f64 * acc;			// [1.0e+12 m/s^2]
 	
-	f64 * mass;
+	f64 * mass;			// [kg/mol]
 	
 	char * locked;
 	int num_locked;
 	
-	f64 tstep1;	// timestep
-	f64 tstep2;	// timestep ^ 2
+	f64 tstep1;		// timestep [fs]
+	f64 tstep2;		// timestep ^ 2
 	
 	f64 ekin;
 	f64 epot;
 	
 	i32s step_counter;
 	
+	f64 sum_of_masses;	// this is for density...
+	
+	friend void model::WriteTrajectoryFrame(ofstream &, moldyn *);
+	
 	public:
 	
-	f64 temperature;
+	f64 target_temperature;		// [K]
+	f64 temperature_rtime;		// [fs]
 	
-	f64 temperature_coupling;
+	f64 target_pressure;		// [bar]
+	f64 pressure_rtime;		// [fs]
+	f64 isoth_compr;		// [1/bar]
+	
+	f64 saved_pressure;
+	f64 saved_density;
 	
 	public:
 	
@@ -123,9 +157,9 @@ class moldyn
 	f64 GetEKin(void) { return ekin; }
 	f64 GetEPot(void) { return epot; }
 	
-	virtual void TakeMDStep(bool);
+	virtual void TakeMDStep(bool, bool);
 	
-	f64 KineticEnergy(void);
+	f64 KineticEnergy(f64 * = NULL);
 	
 	f64 ConvTempEKin(f64);
 	f64 ConvEKinTemp(f64);
@@ -172,22 +206,6 @@ class moldyn_langevin : public moldyn
 
 /*################################################################################################*/
 
-/**	Estimates pressure in a cubic volume element.
-*/
-
-class temperature_meter
-{
-};
-
-/**	Estimates pressure in a cubic volume element.
-*/
-
-class pressure_meter
-{
-};
-
-/*################################################################################################*/
-
 #endif	// MOLDYN_H
 
 // eof
