@@ -10,7 +10,7 @@
 #ifndef RESONANCE_H
 #define RESONANCE_H
 
-#include "libconfig.h"
+//#include "libghemicalconfig2.h"
 
 class resonance_structures;
 
