@@ -13,7 +13,7 @@
 #include "geomopt.h"
 #include "utility.h"
 
-#include <strstream>
+#include <sstream>
 using namespace std;
 
 /*################################################################################################*/
@@ -50,6 +50,8 @@ random_search::random_search(model * p1, i32s p2, i32s p3, i32s p4, i32s p5, i32
 	
 	last_step = NOT_DEFINED;
 	last_E = NOT_DEFINED;
+	
+	srand(time(NULL));
 }
 
 random_search::~random_search(void)
@@ -110,10 +112,10 @@ i32s random_search::TakeStep(void)
 				min_energy = eng->energy;
 			}
 			
-// do not make any output here because that might not be thread-safe...
-//char mbuff1[256]; strstream str1(mbuff1, sizeof(mbuff1));
-//str1 << "step " << (counter1 + 1) << "/" << cycles << "   energy = " << eng->energy << " kJ/mol" << endl << ends;
-//mdl->PrintToLog(mbuff1);
+			ostringstream str1;
+			str1 << "step " << (counter1 + 1) << "/" << cycles << "   energy = " << eng->energy << " kJ/mol" << endl << ends;
+			mdl->PrintToLog(str1.str().c_str());
+			
 			last_step = (counter1 + 1);
 			last_E = eng->energy;
 			
@@ -235,13 +237,13 @@ i32s systematic_search::TakeStep(void)
 				min_energy = eng->energy;
 			}
 			
-if (counter1 != NULL)
-{
-	char mbuff1[256]; strstream str1(mbuff1, sizeof(mbuff1));
-	str1 << "step "; for (int v = 0;v < nvar;v++) { str1 << (char) ('A' + counter1[(nvar - 1) - v]); }
-	str1 << "   energy = " << eng->energy << " kJ/mol" << endl << ends;
-	mdl->PrintToLog(mbuff1);
-}
+			if (counter1 != NULL)
+			{
+				stringstream str1;
+				str1 << "step "; for (int v = 0;v < nvar;v++) { str1 << (char) ('A' + counter1[(nvar - 1) - v]); }
+				str1 << "   energy = " << eng->energy << " kJ/mol" << endl << ends;
+				mdl->PrintToLog(str1.str().c_str());
+			}
 			
 			counter2 = NOT_DEFINED;
 		}
@@ -294,6 +296,8 @@ monte_carlo_search::monte_carlo_search(model * p1, i32s p2, i32s p3, i32s p4, i3
 	
 	CopyCRD(mdl, eng, in_crdset); CopyCRD(eng, mdl, out_crdset);
 	eng->Compute(0); curr_energy = min_energy = eng->energy;
+	
+	srand(time(NULL));
 }
 
 monte_carlo_search::~monte_carlo_search(void)
@@ -375,21 +379,21 @@ i32s monte_carlo_search::TakeStep(void)
 				
 				curr_energy = eng->energy;
 				
-	char mbuff1[256]; strstream str1(mbuff1, sizeof(mbuff1));
-	str1 << "step " << (counter1 + 1) << "/" << n_simul_steps << "   energy = " << eng->energy << " kJ/mol" << endl << ends;
-	mdl->PrintToLog(mbuff1);
+				stringstream str1;
+				str1 << "step " << (counter1 + 1) << "/" << n_simul_steps << "   energy = " << eng->energy << " kJ/mol" << endl << ends;
+				mdl->PrintToLog(str1.str().c_str());
 				
-// set the weight???
-// set the weight???
-// set the weight???
+				// set the weight???
+				// set the weight???
+				// set the weight???
 			}
 			else
 			{
 				counter1--;
 				
-// increase the weighting factor???
-// increase the weighting factor???
-// increase the weighting factor???
+				// increase the weighting factor???
+				// increase the weighting factor???
+				// increase the weighting factor???
 			}
 			
 			counter2 = NOT_DEFINED;
@@ -712,9 +716,11 @@ transition_state_search::transition_state_search(model * p1, f64 p2, f64 p3)
 		
 		patoms.push_back(index);
 		
-		char txtbuff[256]; ostrstream txts(txtbuff, sizeof(txtbuff));
+		ostringstream txts;
 		txts << "no patoms found; using " << index << " as a default." << endl << ends;
-		mdl->PrintToLog(txtbuff); cout << txtbuff;
+		mdl->PrintToLog(txts.str().c_str());
+		
+		cout << txts.str().c_str();
 	}
 	
 	// refine the initial structures + other initializations.
