@@ -20,86 +20,59 @@ using namespace std;
 
 /*################################################################################################*/
 
-chn_info::chn_info(void)
+sb_chain_descriptor::sb_chain_descriptor(bool m1l)
 {
-	type = chn_info::not_defined;
+	mode_1_letter = m1l;
 	
-	id_mol = NOT_DEFINED;
-	id_chn = NOT_DEFINED;
+	seq1 = NULL;
+	seq3 = NULL;
 	
-	length = NOT_DEFINED;
+	if (mode_1_letter) seq1 = new vector<char>;
+	else seq3 = new vector<char *>;
 	
-	sequence = NULL;
 	
-	ss_state = NULL;
-	p_state = NULL;
-	
-	description = NULL;
 }
 
-chn_info::chn_info(chn_type p1, i32s p2)
+sb_chain_descriptor::~sb_chain_descriptor(void)
 {
-	type = p1;
+	if (seq1 != NULL)
+	{
+		delete seq1;
+		seq1 = NULL;
+	}
 	
-	id_mol = NOT_DEFINED;
-	id_chn = NOT_DEFINED;
-	
-	length = p2;
-	
-	sequence = new char[length];
-	
-	ss_state = NULL;
-	p_state = NULL;
-	
-	description = NULL;
+	if (seq3 != NULL)
+	{
+		for (int n1 = 0;n1 < (int) seq3->size();n1++)
+		{
+			char * tmps = seq3->operator[](n1);
+			delete[] tmps;
+		}
+		
+		delete seq3;
+		seq3 = NULL;
+	}
 }
 
-chn_info::chn_info(const chn_info & p1)
+int sb_chain_descriptor::AddRes1(char r1l)
 {
-	type = p1.type;
-	
-	id_mol = p1.id_mol;
-	id_chn = p1.id_chn;
-	
-	length = p1.length;
-	
-	if (p1.sequence != NULL)
-	{
-		sequence = new char[length];
-		for (i32s n1 = 0;n1 < length;n1++) sequence[n1] = p1.sequence[n1];
-	}
-	else sequence = NULL;
-	
-	if (p1.ss_state != NULL)
-	{
-		ss_state = new char[length];
-		for (i32s n1 = 0;n1 < length;n1++) ss_state[n1] = p1.ss_state[n1];
-	}
-	else ss_state = NULL;
-
-	if (p1.p_state != NULL)
-	{
-		p_state = new char[length];
-		for (i32s n1 = 0;n1 < length;n1++) p_state[n1] = p1.p_state[n1];
-	}
-	else p_state = NULL;
-	
-	if (p1.description != NULL)
-	{
-		description = new char[strlen(p1.description) + 1];
-		strcpy(description, p1.description);
-	}
-	else description = NULL;
+	seq1->push_back(r1l);
+	return (int) seq1->size();
 }
 
-chn_info::~chn_info(void)
+int sb_chain_descriptor::AddRes3(const char * r3l)
 {
-	if (sequence != NULL) delete[] sequence;
+	if (strlen(r3l) != 3)
+	{
+		cout << "sb_chain_descriptor::AddRes3() ; bad input" << endl;
+		exit(EXIT_FAILURE);
+	}
 	
-	if (ss_state != NULL) delete[] ss_state;
-	if (p_state != NULL) delete[] p_state;
+	char * tmps = new char[4];
+	strcpy(tmps, r3l);
 	
-	if (description != NULL) delete[] description;
+	seq3->push_back(tmps);
+	return (int) seq3->size();
 }
 
 /*################################################################################################*/
@@ -165,13 +138,13 @@ sequencebuilder::sequencebuilder(chn_info::chn_type ct, char * filename)
 	
 	// read the default modifications for residues...
 	
-	while (file.peek() != 'B') file.getline(buffer, sizeof(buffer));					// default...
+	while (file.peek() != 'B') file.getline(buffer, sizeof(buffer));	// default...
 	file.getline(buffer, sizeof(buffer)); mod[0] = new sb_data_res; mod[0]->ReadModification(file);
 	
-	while (file.peek() != 'H') file.getline(buffer, sizeof(buffer));					// secondary!
+	while (file.peek() != 'H') file.getline(buffer, sizeof(buffer));	// head-mods
 	file.getline(buffer, sizeof(buffer)); mod[1] = new sb_data_res; mod[1]->ReadModification(file);
 	
-	while (file.peek() != 'T') file.getline(buffer, sizeof(buffer));					// primary!
+	while (file.peek() != 'T') file.getline(buffer, sizeof(buffer));	// tail-mods
 	file.getline(buffer, sizeof(buffer)); mod[2] = new sb_data_res; mod[2]->ReadModification(file);
 	
 	// read descriptions for the residues...
@@ -181,7 +154,7 @@ sequencebuilder::sequencebuilder(chn_info::chn_type ct, char * filename)
 		if (file.peek() == 'R')
 		{
 			sb_data_res newres; file >> newres;
-			residue_vector.push_back(newres);
+			resi_vector.push_back(newres);
 		}
 		else file.getline(buffer, sizeof(buffer));
 	}
@@ -200,17 +173,38 @@ sequencebuilder::~sequencebuilder(void)
 
 // here we use only the first coordinate set, no matter how many there are...
 
-void sequencebuilder::Build(model * mdl, char * sequence, f64 * tor)
+// char * sequence, f64 * tor
+void sequencebuilder::Build(model * mdl, sb_chain_descriptor * chde)
 {
-	fGL b1crd[3] = { 1.0, 1.0, 0.0 };	// these should be const...
+	bool bad_chde = false;
+	if (chde->seq1 == NULL && chde->seq3 == NULL) bad_chde = true;
+	if (chde->mode_1_letter)
+	{
+		if (chde->seq1 == NULL) bad_chde = true;
+	}
+	else
+	{
+		if (chde->seq3 == NULL) bad_chde = true;
+	}
+	
+	if (bad_chde)
+	{
+		cout << "sequencebuilder::Build() ; bad input" << endl;
+		exit(EXIT_FAILURE);
+	}
+	
+	const int num_res = (chde->mode_1_letter ? chde->seq1->size() : chde->seq3->size());
+	const int num_cs = mdl->GetCRDSetCount();
+	
+	fGL b1crd[3] = { 1.0, 1.0, 0.0 };		// these should be const...
 	fGL b2crd[3] = { 1.0, 0.0, 0.0 };
 	fGL b3crd[3] = { 0.0, 0.0, 0.0 };
 	
 	atom base[3] =
 	{
-		atom(element(), b1crd, mdl->cs_vector.size()),		// these should be const...
-		atom(element(), b2crd, mdl->cs_vector.size()),
-		atom(element(), b3crd, mdl->cs_vector.size())
+		atom(element(), b1crd, num_cs),		// these should be const...
+		atom(element(), b2crd, num_cs),
+		atom(element(), b3crd, num_cs)
 	};
 	
 	id_vector.resize(0); ref_vector.resize(0);
@@ -218,10 +212,18 @@ void sequencebuilder::Build(model * mdl, char * sequence, f64 * tor)
 	id_vector.push_back(main_vector[0].prev[1]); ref_vector.push_back(& base[1]);
 	id_vector.push_back(main_vector[0].prev[2]); ref_vector.push_back(& base[2]);
 	
-	for (i32u n1 = 0;n1 < strlen(sequence);n1++)
+	for (i32u n1 = 0;n1 < num_res;n1++)
 	{
 		bool first_res = (n1 == 0);
-		bool last_res = (n1 == (strlen(sequence) - 1));
+		bool last_res = (n1 == (num_res - 1));
+		
+		vector<f64> tor_v;
+		for (i32u n2 = 0;n2 < chde->default_torsions.size();n2++)
+		{
+			tor_v.push_back(chde->default_torsions[n2]);
+		}
+		
+		// build main chain.
 		
 		for (i32u n2 = 0;n2 < main_vector.size();n2++)
 		{
@@ -233,22 +235,26 @@ void sequencebuilder::Build(model * mdl, char * sequence, f64 * tor)
 				prev[n3] = ref_vector[n4]; ic[n3] = main_vector[n2].ic1[n3];
 			}
 			
-			if (main_vector[n2].ic2 > -1) ic[2] += tor[main_vector[n2].ic2];
+			const int tor_i = main_vector[n2].ic2;
+			if (tor_i > -1 && tor_i < tor_v.size()) ic[2] += tor_v[tor_i];
+			
 			Convert(prev, ic, crd);
 			
-			id_vector.push_back(main_vector[n2].id[0]);
-			atom newatom(main_vector[n2].el, crd, mdl->cs_vector.size());
+			id_vector.push_back(main_vector[n2].id);
+			atom newatom(main_vector[n2].el, crd, num_cs);
 			mdl->AddAtom(newatom);
 			
-			ref_vector.push_back(& mdl->atom_list.back());
+			ref_vector.push_back(& mdl->GetLastAtom());
 			
 			if (!first_res || n2 != 0)
 			{
-				bond newbond(prev[0], & mdl->atom_list.back(), main_vector[n2].bt[0]);
+				bond newbond(prev[0], & mdl->GetLastAtom(), main_vector[n2].bt);
 				mdl->AddBond(newbond);
 			}
 		}
 		
+		// build connection (if needed).
+		
 		if (!last_res)
 		{
 			for (i32u n2 = 0;n2 < conn_vector.size();n2++)
@@ -261,42 +267,109 @@ void sequencebuilder::Build(model * mdl, char * sequence, f64 * tor)
 					prev[n3] = ref_vector[n4]; ic[n3] = conn_vector[n2].ic1[n3];
 				}
 				
-				if (conn_vector[n2].ic2 > -1) ic[2] += tor[conn_vector[n2].ic2];
+				const int tor_i = conn_vector[n2].ic2;
+				if (tor_i > -1 && tor_i < tor_v.size()) ic[2] += tor_v[tor_i];
+				
 				Convert(prev, ic, crd);
 				
-				id_vector.push_back(conn_vector[n2].id[0]);
-				atom newatom(conn_vector[n2].el, crd, mdl->cs_vector.size());
+				id_vector.push_back(conn_vector[n2].id);
+				atom newatom(conn_vector[n2].el, crd, num_cs);
 				mdl->AddAtom(newatom);
 				
-				ref_vector.push_back(& mdl->atom_list.back());
+				ref_vector.push_back(& mdl->GetLastAtom());
 				
-				bond newbond(prev[0], & mdl->atom_list.back(), conn_vector[n2].bt[0]);
+				bond newbond(prev[0], & mdl->GetLastAtom(), conn_vector[n2].bt);
 				mdl->AddBond(newbond);
 			}
 		}
 		
-		if (last_res) Build(mdl, mod[2], tor);		// primary!
-		else if (first_res) Build(mdl, mod[1], tor);	// secondary!
-		else Build(mdl, mod[0], tor);			// default...
+		if (last_res) BuildResidue(mdl, mod[2], tor_v);		// tail-mods
+		else if (first_res)					// head-mods
+		{
+			bool first_is_proline = false;
+			
+			if (type == chn_info::amino_acid)
+			{
+				if (chde->mode_1_letter)
+				{
+					if (chde->seq1->operator[](n1) == 'P') first_is_proline = true;
+				}
+				else
+				{
+					if (!strcmp("PRO", chde->seq3->operator[](n1))) first_is_proline = true;
+				}
+			}
+			
+			if (!first_is_proline)
+			{
+				BuildResidue(mdl, mod[1], tor_v);
+			}
+			else
+			{
+				sb_data_res tmpmod(* mod[1]);
+				if (tmpmod.atm_vector.size() != 4)
+				{
+					cout << "check the builder/amino.txt file" << endl;
+					exit(EXIT_FAILURE);
+				}
+				
+				// modify the torsions:
+				// ^^^^^^^^^^^^^^^^^^^^
+				
+				tmpmod.atm_vector[2].ic1[2] = M_PI * 000.0 / 180.0;
+				tmpmod.atm_vector[3].ic1[2] = M_PI * 220.0 / 180.0;
+				
+				BuildResidue(mdl, & tmpmod, tor_v);
+			}
+		}
+		else BuildResidue(mdl, mod[0], tor_v);			// default...
 		
 		i32u res = 0;
-		while (residue_vector[res].symbol != sequence[n1])
+		while (true)
 		{
-			if (res != residue_vector.size()) res++;
+			if (chde->mode_1_letter)
+			{
+				if (resi_vector[res].symbol1 == chde->seq1->operator[](n1)) break;
+			}
+			else
+			{
+				const char * seq3str = chde->seq3->operator[](n1);
+				if (!strcmp(resi_vector[res].symbol3, seq3str)) break;
+			}
+			
+			if (res != resi_vector.size()) res++;
 			else break;
 		}
 		
-		if (res != residue_vector.size()) Build(mdl, & residue_vector[res], tor);
-		else cout << "unknown residue " << sequence[n1] << endl;
+		if (res != resi_vector.size()) BuildResidue(mdl, & resi_vector[res], tor_v);
+		else
+		{
+			cout << "unknown residue ";
+			
+			if (chde->mode_1_letter)
+			{
+				cout << chde->seq1->operator[](n1);
+			}
+			else
+			{
+				cout << chde->seq3->operator[](n1);
+			}
+			
+			cout << endl;
+		}
 		
 		vector<i32s> tmpv1; atmr_vector tmpv2;
 		for (i32u n2 = 0;n2 < id_vector.size();n2++)
 		{
 			if (id_vector[n2] & 0xFF00) continue;
-			tmpv1.push_back(id_vector[n2]); tmpv2.push_back(ref_vector[n2]);
+			
+			tmpv1.push_back(id_vector[n2]);
+			tmpv2.push_back(ref_vector[n2]);
 		}
 		
-		id_vector.resize(0); ref_vector.resize(0);
+		id_vector.resize(0);
+		ref_vector.resize(0);
+		
 		for (i32u n2 = 0;n2 < tmpv1.size();n2++)
 		{
 			id_vector.push_back(tmpv1[n2] + 0xFF00);
@@ -367,8 +440,11 @@ cout << "nmol = " << mdl->nmol << endl;
 			}
 		}
 		
-		cout << "found " << head_atoms.size() << " possible heads and ";
-		cout << tail_atoms.size() << " possible tails." << endl;
+		if (head_atoms.size() > 0 && tail_atoms.size() > 0)
+		{
+			cout << "found " << head_atoms.size() << " possible heads and ";
+			cout << tail_atoms.size() << " possible tails." << endl;
+		}
 		
 		// now we have all possible head/tail atoms. next we have to check
 		// all possible paths between them to find all possible main chains...
@@ -382,7 +458,10 @@ cout << "nmol = " << mdl->nmol << endl;
 			}
 		}
 		
-		if (path_vector.size()) cout << path_vector.size() << " chains:" << endl;
+		if (path_vector.size())
+		{
+			cout << path_vector.size() << " chains:" << endl;
+		}
 		
 		for (i32s n2 = 0;n2 < (i32s) path_vector.size();n2++)
 		{
@@ -393,9 +472,8 @@ cout << "nmol = " << mdl->nmol << endl;
 				(* it1).bndr->flags[0] = true;
 			}
 			
-			vector<char> sequence;
-bool head_blocked = !head_vector[0].Check(mdl, path_vector[n2].front(), 1);	// what exactly is this???
-bool tail_blocked = !tail_vector[0].Check(mdl, path_vector[n2].back(), 1);	// what exactly is this???
+			vector<char> seq1;
+			vector<char *> seq3;
 			
 			i32u acount = 0;	// atom counter
 			i32u rcount = 0;	// residue counter
@@ -406,8 +484,58 @@ bool tail_blocked = !tail_vector[0].Check(mdl, path_vector[n2].back(), 1);	// wh
 				// path_vector). we use it to determine whether we have reached the last
 				// residue, and to increment acount.
 				
-				bool is_first = !head_blocked && !rcount;
-				bool is_last = !tail_blocked && (acount + main_vector.size() + conn_vector.size() >= path_vector[n2].size());
+				int head_el = NOT_DEFINED;
+				int tail_el = NOT_DEFINED;
+				
+				bool is_first = !rcount;
+				if (is_first && type == chn_info::amino_acid)
+				{
+					// an extra test for an ACE-like block group.
+					// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+					
+					atom * refA = path_vector[n2][acount + 0];
+					atom * refB = path_vector[n2][acount + 1];
+					atom * refX = NULL;
+					
+					for (iter_cl itX = refA->cr_list.begin();itX != refA->cr_list.end();itX++)
+					{
+						if ((* itX).atmr->el.GetAtomicNumber() <= 1) continue;
+						if ((* itX).atmr == refB) continue;
+						
+						refX = (* itX).atmr;
+					}
+					
+					if (refX != NULL)
+					{
+						head_el = refX->el.GetAtomicNumber();
+						is_first = false;
+					}
+				}
+				
+				bool is_last = (acount + main_vector.size() + conn_vector.size()) >= path_vector[n2].size();
+				if (is_last && type == chn_info::amino_acid)
+				{
+					// an extra test for an NME-like block group.
+					// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+					
+					atom * refA = path_vector[n2][acount + 2];
+					atom * refB = path_vector[n2][acount + 1];
+					atom * refX = NULL;
+					
+					for (iter_cl itX = refA->cr_list.begin();itX != refA->cr_list.end();itX++)
+					{
+						if ((* itX).bndr->bt.GetValue() != 1) continue;
+						if ((* itX).atmr == refB) continue;
+						
+						refX = (* itX).atmr;
+					}
+					
+					if (refX != NULL)
+					{
+						tail_el = refX->el.GetAtomicNumber();
+						is_last = false;
+					}
+				}
 				
 				i32u rsize = main_vector.size();
 				if (!is_last) rsize += conn_vector.size();
@@ -419,19 +547,52 @@ bool tail_blocked = !tail_vector[0].Check(mdl, path_vector[n2].back(), 1);	// wh
 				// there may not be any missing atoms, and the closest match is
 				// the one with largest number of correct atoms identified.
 				
-// some problems with the blocked head/tail residues???
-				
 				i32s tmp1[2] = { NOT_DEFINED, NOT_DEFINED };
-				for (i32s n4 = 0;n4 < (i32s) residue_vector.size();n4++)
+				for (i32s n4 = 0;n4 < (i32s) resi_vector.size();n4++)
 				{
 					vector<sb_tdata> tdata;
 					BuildTemplate(tdata, n4, is_first, is_last);
 					
+					// do the template mods if blockgroups!!!
+					// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+					
+					if (head_el != NOT_DEFINED)
+					{
+						if (is_first)
+						{
+							cout << "bad head_el" << endl;
+							exit(EXIT_FAILURE);
+						}
+						
+						// ACE needs no modifications...
+					}
+					
+					if (tail_el != NOT_DEFINED)
+					{
+						if (is_last)
+						{
+							cout << "bad tail_el" << endl;
+							exit(EXIT_FAILURE);
+						}
+						
+						for (i32s n5 = 0;n5 < (i32s) tdata.size();n5++)
+						{
+							if (tdata[n5].id[0] == 0x11)
+							{
+								tdata[n5].el = element(tail_el);
+							}
+						}
+					}
+					
+					// set the main chain stuff...
+					
 					for (i32s n5 = 0;n5 < (i32s) main_vector.size();n5++)
 					{
 						tdata[n5].ref = path_vector[n2][acount + n5];
 					}
 					
+					// set the connection stuff...
+					
 					if (!is_last)
 					{
 						const int mvsz = main_vector.size();
@@ -442,6 +603,11 @@ bool tail_blocked = !tail_vector[0].Check(mdl, path_vector[n2].back(), 1);	// wh
 					}
 					
 					bool result = CheckTemplate(tdata, 0);
+///////////////////////////////////////////////////////////////////////////////////////////
+//cout << "checktemplate " << n4 << " " << is_first << " " << is_last << " ; ";
+//cout << "result = " << result << endl;
+//int zzz; cin >> zzz;
+///////////////////////////////////////////////////////////////////////////////////////////
 					if (result && (tmp1[0] == NOT_DEFINED || ((i32s) tdata.size()) > tmp1[1]))
 					{
 						tmp1[0] = n4;
@@ -453,7 +619,12 @@ bool tail_blocked = !tail_vector[0].Check(mdl, path_vector[n2].back(), 1);	// wh
 				{
 					cout << "WARNING : residue " << rcount << " was of unknown type!!!" << endl;
 					
-					sequence.push_back('?');				// handle the sequence...
+					seq1.push_back('?');	// handle the sequence...
+					
+					char * tmps = new char[4];
+					strcpy(tmps, "???");
+					
+					seq3.push_back(tmps);	// handle the sequence...
 					
 					for (i32u n3 = 0;n3 < main_vector.size();n3++)		// handle the numbers...
 					{
@@ -465,7 +636,12 @@ bool tail_blocked = !tail_vector[0].Check(mdl, path_vector[n2].back(), 1);	// wh
 				}
 				else
 				{
-					sequence.push_back(residue_vector[tmp1[0]].symbol);
+					seq1.push_back(resi_vector[tmp1[0]].symbol1);
+					
+					char * tmps = new char[4];
+					strcpy(tmps, resi_vector[tmp1[0]].symbol3);
+					
+					seq3.push_back(tmps);
 					
 					// rebuild the best matching template and use that to
 					// set up the atom ID numbers...
@@ -490,65 +666,72 @@ bool tail_blocked = !tail_vector[0].Check(mdl, path_vector[n2].back(), 1);	// wh
 					CheckTemplate(tdata, 0);
 					for (i32s n4 = 0;n4 < (i32s) tdata.size();n4++)
 					{
-						tdata[n4].ref->builder_res_id = (residue_vector[tmp1[0]].id << 8) + tdata[n4].id[0];
+						tdata[n4].ref->builder_res_id = (resi_vector[tmp1[0]].id << 8) + tdata[n4].id[0];
 						
 						tdata[n4].ref->id[1] = mdl->ref_civ->size();
 						tdata[n4].ref->id[2] = rcount;
 					}
 					
-					// finally do a hard-coded fix for unambiguous TRP residues...
-					// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+/////////////////////////////////////////////////////////////////////////// exceptions
+/////////////////////////////////////////////////////////////////////////// exceptions
+
+// finally do a hard-coded fix for unambiguous TRP residues...
+// ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+
+if (type == chn_info::amino_acid && resi_vector[tmp1[0]].symbol1 == 'W')
+{
+	atom * ref23 = NULL;
+	atom * ref25 = NULL;
+	atom * ref26 = NULL;
+	atom * ref28 = NULL;
+	atom * ref29 = NULL;
+	
+	for (i32s n4 = 0;n4 < (i32s) tdata.size();n4++)
+	{
+		if ((tdata[n4].ref->builder_res_id & 0xFF) == 0x23) ref23 = tdata[n4].ref;
+		if ((tdata[n4].ref->builder_res_id & 0xFF) == 0x25) ref25 = tdata[n4].ref;
+		if ((tdata[n4].ref->builder_res_id & 0xFF) == 0x26) ref26 = tdata[n4].ref;
+		if ((tdata[n4].ref->builder_res_id & 0xFF) == 0x28) ref28 = tdata[n4].ref;
+		if ((tdata[n4].ref->builder_res_id & 0xFF) == 0x29) ref29 = tdata[n4].ref;
+	}
+	
+	if (!ref23) { cout << "TRP-fix : ref23 not found!" << endl; exit(EXIT_FAILURE); }
+	if (!ref25) { cout << "TRP-fix : ref25 not found!" << endl; exit(EXIT_FAILURE); }
+	if (!ref26) { cout << "TRP-fix : ref26 not found!" << endl; exit(EXIT_FAILURE); }
+	if (!ref28) { cout << "TRP-fix : ref28 not found!" << endl; exit(EXIT_FAILURE); }
+	if (!ref29) { cout << "TRP-fix : ref29 not found!" << endl; exit(EXIT_FAILURE); }
+	
+	bond b1 = bond(ref23, ref25, bondtype('S'));
+	iter_bl itb1 = find(mdl->GetBondsBegin(), mdl->GetBondsEnd(), b1);
+	bool b1exists = (itb1 != mdl->GetBondsEnd());
+	
+	bond b2 = bond(ref23, ref29, bondtype('S'));
+	iter_bl itb2 = find(mdl->GetBondsBegin(), mdl->GetBondsEnd(), b2);
+	bool b2exists = (itb2 != mdl->GetBondsEnd());
+	
+	if (b1exists == b2exists) { cout << "TRP-fix error #1" << endl; exit(EXIT_FAILURE); }
+	if (b1exists && !b2exists)
+	{
+		cout << "TRP-fix!!!" << endl;
+		
+		// ok, this seems to be "wrong" way...
+		// fix by exhanging 25<->29 and 26<->28.
+		
+		int tmpid;
+		
+		tmpid = ref25->builder_res_id;
+		ref25->builder_res_id = ref29->builder_res_id;
+		ref29->builder_res_id = tmpid;
+		
+		tmpid = ref26->builder_res_id;
+		ref26->builder_res_id = ref28->builder_res_id;
+		ref28->builder_res_id = tmpid;
+	}
+}
+
+/////////////////////////////////////////////////////////////////////////// exceptions
+/////////////////////////////////////////////////////////////////////////// exceptions
 					
-					if (type == chn_info::amino_acid && residue_vector[tmp1[0]].symbol == 'W')
-					{
-						atom * ref23 = NULL;
-						atom * ref25 = NULL;
-						atom * ref26 = NULL;
-						atom * ref28 = NULL;
-						atom * ref29 = NULL;
-						
-						for (i32s n4 = 0;n4 < (i32s) tdata.size();n4++)
-						{
-							if ((tdata[n4].ref->builder_res_id & 0xFF) == 0x23) ref23 = tdata[n4].ref;
-							if ((tdata[n4].ref->builder_res_id & 0xFF) == 0x25) ref25 = tdata[n4].ref;
-							if ((tdata[n4].ref->builder_res_id & 0xFF) == 0x26) ref26 = tdata[n4].ref;
-							if ((tdata[n4].ref->builder_res_id & 0xFF) == 0x28) ref28 = tdata[n4].ref;
-							if ((tdata[n4].ref->builder_res_id & 0xFF) == 0x29) ref29 = tdata[n4].ref;
-						}
-						
-						if (!ref23) { cout << "TRP-fix : ref23 not found!" << endl; exit(EXIT_FAILURE); }
-						if (!ref25) { cout << "TRP-fix : ref25 not found!" << endl; exit(EXIT_FAILURE); }
-						if (!ref26) { cout << "TRP-fix : ref26 not found!" << endl; exit(EXIT_FAILURE); }
-						if (!ref28) { cout << "TRP-fix : ref28 not found!" << endl; exit(EXIT_FAILURE); }
-						if (!ref29) { cout << "TRP-fix : ref29 not found!" << endl; exit(EXIT_FAILURE); }
-						
-						bond b1 = bond(ref23, ref25, bondtype('S'));
-						iter_bl itb1 = find(mdl->GetBondsBegin(), mdl->GetBondsEnd(), b1);
-						bool b1exists = (itb1 != mdl->GetBondsEnd());
-						
-						bond b2 = bond(ref23, ref29, bondtype('S'));
-						iter_bl itb2 = find(mdl->GetBondsBegin(), mdl->GetBondsEnd(), b2);
-						bool b2exists = (itb2 != mdl->GetBondsEnd());
-						
-						if (b1exists == b2exists) { cout << "TRP-fix error #1" << endl; exit(EXIT_FAILURE); }
-						if (b1exists && !b2exists)
-						{
-							cout << "TRP-fix!!!" << endl;
-							
-							// ok, this seems to be "wrong" way...
-							// fix by exhanging 25<->29 and 26<->28.
-							
-							int tmpid;
-							
-							tmpid = ref25->builder_res_id;
-							ref25->builder_res_id = ref29->builder_res_id;
-							ref29->builder_res_id = tmpid;
-							
-							tmpid = ref26->builder_res_id;
-							ref26->builder_res_id = ref28->builder_res_id;
-							ref28->builder_res_id = tmpid;
-						}
-					}
 				}
 				
 				rcount++;
@@ -557,12 +740,22 @@ bool tail_blocked = !tail_vector[0].Check(mdl, path_vector[n2].back(), 1);	// wh
 			
 			// next we will write down the sequence...
 			
-			chn_info newinfo(type, sequence.size());
+			if (seq1.size() != seq3.size())
+			{
+				cout << "the seq1/seq3 arrays are bad!" << endl;
+				exit(EXIT_FAILURE);
+			}
+			
+			chn_info newinfo(type, seq1.size());
 			
 			newinfo.id_mol = n1;
 			newinfo.id_chn = mdl->ref_civ->size();
 			
-			for (i32u n3 = 0;n3 < sequence.size();n3++) newinfo.sequence[n3] = sequence[n3];
+			for (i32u n3 = 0;n3 < seq1.size();n3++)
+			{
+				newinfo.sequence1[n3] = seq1[n3];	// a char...
+				newinfo.sequence3[n3] = seq3[n3];	// an array...
+			}
 			
 			mdl->ref_civ->push_back(newinfo);
 			
@@ -571,12 +764,12 @@ bool tail_blocked = !tail_vector[0].Check(mdl, path_vector[n2].back(), 1);	// wh
 			cout << "> chain " << newinfo.id_chn;
 			cout << ", length " << newinfo.length << ":" << endl;
 			
-			for (i32u n3 = 0;n3 < sequence.size();n3++)
+			for (i32u n3 = 0;n3 < seq1.size();n3++)
 			{
-				cout << sequence[n3];
+				cout << seq1[n3];
 				
 				bool is_break = !((n3 + 1) % 50);
-				bool is_end = ((n3 + 1) == sequence.size());
+				bool is_end = ((n3 + 1) == seq1.size());
 				
 				if (is_break || is_end) cout << endl;
 			}
@@ -590,10 +783,10 @@ bool tail_blocked = !tail_vector[0].Check(mdl, path_vector[n2].back(), 1);	// wh
 				(* it1).bndr->flags[0] = false;
 			}
 		}
-		
-		// now check the HYDROGENS that are ignored by the seqbuilder generally.
-		// for hydrogens, set the same id's that the heavy atom has if they are valid ones.
-		
+// obsolete obsolete obsolete obsolete obsolete obsolete obsolete obsolete obsolete obsolete
+// obsolete obsolete obsolete obsolete obsolete obsolete obsolete obsolete obsolete obsolete
+	/*	now check the HYDROGENS that are ignored by the seqbuilder generally.
+		for hydrogens, set the same id's that the heavy atom has if they are valid ones.
 		for (iter_al it1 = range[0];it1 != range[1];it1++)
 		{
 			if ((* it1).el.GetAtomicNumber() != 1) continue;
@@ -612,7 +805,9 @@ bool tail_blocked = !tail_vector[0].Check(mdl, path_vector[n2].back(), 1);	// wh
 			
 			(* it1).id[1] = heavy->id[1];
 			(* it1).id[2] = heavy->id[2];
-		}
+		}	*/
+// obsolete obsolete obsolete obsolete obsolete obsolete obsolete obsolete obsolete obsolete
+// obsolete obsolete obsolete obsolete obsolete obsolete obsolete obsolete obsolete obsolete
 	}
 	
 	// traditionally, we sorted the atom list here to make the newly assigned chains in nice order.
@@ -620,8 +815,10 @@ bool tail_blocked = !tail_vector[0].Check(mdl, path_vector[n2].back(), 1);	// wh
 	// objects (like for amino/nucleic acid variants) and we like to use them all...
 }
 
-void sequencebuilder::Build(model * mdl, sb_data_res * res, f64 * tor)
+void sequencebuilder::BuildResidue(model * mdl, sb_data_res * res, vector<f64> & tor_v)
 {
+	const int num_cs = mdl->GetCRDSetCount();
+	
 	for (i32u n1 = 0;n1 < res->atm_vector.size();n1++)
 	{
 		atom * prev[3];
@@ -632,16 +829,18 @@ void sequencebuilder::Build(model * mdl, sb_data_res * res, f64 * tor)
 			prev[n2] = ref_vector[n3]; ic[n2] = res->atm_vector[n1].ic1[n2];
 		}
 		
-		if (res->atm_vector[n1].ic2 > -1) ic[2] += tor[res->atm_vector[n1].ic2];
+		const int tor_i = res->atm_vector[n1].ic2;
+		if (tor_i > -1 && tor_i < tor_v.size()) ic[2] += tor_v[tor_i];
+		
 		Convert(prev, ic, crd);
 		
-		id_vector.push_back(res->atm_vector[n1].id[0]);
-		atom newatom(res->atm_vector[n1].el, crd, mdl->cs_vector.size());
+		id_vector.push_back(res->atm_vector[n1].id);
+		atom newatom(res->atm_vector[n1].el, crd, num_cs);
 		mdl->AddAtom(newatom);
 		
-		ref_vector.push_back(& mdl->atom_list.back());
+		ref_vector.push_back(& mdl->GetLastAtom());
 		
-		bond newbond(prev[0], & mdl->atom_list.back(), res->atm_vector[n1].bt[0]);
+		bond newbond(prev[0], & mdl->GetLastAtom(), res->atm_vector[n1].bt);
 		mdl->AddBond(newbond);
 	}
 	
@@ -650,6 +849,7 @@ void sequencebuilder::Build(model * mdl, sb_data_res * res, f64 * tor)
 		atom * tmp1[2]; i32s tmp2;
 		tmp2 = 0; while (id_vector[tmp2] != res->bnd_vector[n1].atm[0]) tmp2++; tmp1[0] = ref_vector[tmp2];
 		tmp2 = 0; while (id_vector[tmp2] != res->bnd_vector[n1].atm[1]) tmp2++; tmp1[1] = ref_vector[tmp2];
+		
 		bond newbond(tmp1[0], tmp1[1], res->bnd_vector[n1].bt);
 		mdl->AddBond(newbond);
 	}
@@ -673,6 +873,36 @@ void sequencebuilder::Convert(atom * prev[], f64 * ic, fGL * crd)
 	for (i32s n1 = 0;n1 < 3;n1++) crd[n1] = tmp3.data[n1];
 }
 
+void sequencebuilder::BuildTemplate(vector<sb_tdata> & tdata, i32s res, bool is_first, bool is_last)
+{
+	BuildPartialT(tdata, main_vector);
+	if (!is_last) BuildPartialT(tdata, conn_vector);
+	
+	if (is_last) BuildPartialT(tdata, mod[2]->atm_vector);		// tail-mods
+	else if (is_first) BuildPartialT(tdata, mod[1]->atm_vector);	// head-mods
+	else BuildPartialT(tdata, mod[0]->atm_vector);			// default...
+	
+	BuildPartialT(tdata, resi_vector[res].atm_vector);
+}
+
+void sequencebuilder::BuildPartialT(vector<sb_tdata> & tdata, vector<sb_data_atm> & adata)
+{
+	for (i32u n1 = 0;n1 < adata.size();n1++)
+	{
+		sb_tdata newdata;
+		newdata.id[0] = adata[n1].id;
+		
+		if (adata[n1].prev[0] & 0xFF00) newdata.id[1] = NOT_DEFINED;
+		else newdata.id[1] = adata[n1].prev[0];
+		
+		newdata.el = adata[n1].el;
+		newdata.bt = adata[n1].bt;
+		newdata.ref = NULL;
+		
+		tdata.push_back(newdata);
+	}
+}
+
 // idea is that this path-searching function should be used only inside a molecule.
 // this way we are sure to find always at least one path. we collect references to
 // atoms for all paths and collect this way all possible paths.
@@ -698,10 +928,10 @@ void sequencebuilder::FindPath(model * mdl, atom * ref1, atom * ref2, i32u index
 	
 	temporary_vector.push_back(ref1);
 	
-//	for (i32u debug = 0;debug < temporary_vector.size();debug++)
-//	{
-//		cout << temporary_vector[debug]->el.GetSymbol();
-//	}	cout << endl;
+//for (i32u debug = 0;debug < temporary_vector.size();debug++)
+//{
+//	cout << temporary_vector[debug]->el.GetSymbol();
+//}	cout << endl;
 	
 	if (ref1 == ref2) path_vector.push_back(temporary_vector);
 	else
@@ -719,36 +949,6 @@ void sequencebuilder::FindPath(model * mdl, atom * ref1, atom * ref2, i32u index
 	temporary_vector.pop_back();
 }
 
-void sequencebuilder::BuildTemplate(vector<sb_tdata> & tdata, i32s res, bool is_first, bool is_last)
-{
-	BuildTemplate(tdata, main_vector);
-	if (!is_last) BuildTemplate(tdata, conn_vector);
-	
-	if (is_last) BuildTemplate(tdata, mod[2]->atm_vector);		// primary!
-	else if (is_first) BuildTemplate(tdata, mod[1]->atm_vector);	// secondary!
-	else BuildTemplate(tdata, mod[0]->atm_vector);			// default...
-	
-	BuildTemplate(tdata, residue_vector[res].atm_vector);
-}
-
-void sequencebuilder::BuildTemplate(vector<sb_tdata> & tdata, vector<sb_data_atm> & adata)
-{
-	for (i32u n1 = 0;n1 < adata.size();n1++)
-	{
-		sb_tdata newdata;
-		newdata.id[0] = adata[n1].id[0];
-		
-		if (adata[n1].prev[0] & 0xFF00) newdata.id[1] = NOT_DEFINED;
-		else newdata.id[1] = adata[n1].prev[0];
-		
-		newdata.el = adata[n1].el;
-		newdata.bt = adata[n1].bt[1];
-		newdata.ref = NULL;
-		
-		tdata.push_back(newdata);
-	}
-}
-
 bool sequencebuilder::CheckTemplate(vector<sb_tdata> & tdata, i32s flag)
 {
 	vector<i32s> tmpv1;
@@ -789,11 +989,10 @@ bool sequencebuilder::CheckTemplate(vector<sb_tdata> & tdata, i32s flag)
 	for (it1 = tdata[index].ref->cr_list.begin();it1 != tdata[index].ref->cr_list.end();it1++)
 	{
 		bool test1 = (* it1).bndr->flags[flag];
-		bool test2 = ((* it1).atmr->el.GetAtomicNumber() != 1);
-		if (!test1 && test2) tmpv2.push_back((* it1));
+		if (!test1) tmpv2.push_back((* it1));
 	}
 	
-	// if there is less bond that we need, the template can't match -> FALSE!!!
+	// if there are less bonds than we need, the template can't match -> FALSE!!!
 	
 	if (tmpv2.size() < tmpv1.size()) return false;
 	
@@ -852,18 +1051,21 @@ sb_data_atm::sb_data_atm(void)
 
 sb_data_atm::sb_data_atm(const sb_data_atm & p1)
 {
-	el = p1.el; ic2 = p1.ic2;
-	bt[0] = p1.bt[0]; bt[1] = p1.bt[1];
-	id[0] = p1.id[0]; id[1] = p1.id[1];
+	id = p1.id;
 	
-	if (p1.tr != NULL) tr = new typerule(* p1.tr);
-	else tr = NULL;
+	el = p1.el;
+	bt = p1.bt;
+	
+	ic2 = p1.ic2;
 	
 	for (i32s n1 = 0;n1 < 3;n1++)
 	{
 		prev[n1] = p1.prev[n1];
 		ic1[n1] = p1.ic1[n1];
 	}
+	
+	if (p1.tr != NULL) tr = new typerule(* p1.tr);
+	else tr = NULL;
 }
 
 sb_data_atm::~sb_data_atm(void)
@@ -875,7 +1077,7 @@ istream & operator>>(istream & p1, sb_data_atm & p2)
 {
 	char buffer[256];
 	while (p1.get() != 'M');
-	p1 >> p2.id[0]; while (p1.get() != ':');
+	p1 >> p2.id; while (p1.get() != ':');
 	p1 >> buffer; p2.el = element(buffer);
 	p1 >> p2.prev[0] >> p2.prev[1] >> p2.prev[2];
 	p1 >> p2.ic1[0] >> p2.ic1[1] >> p2.ic2 >> p2.ic1[2];
@@ -884,10 +1086,11 @@ istream & operator>>(istream & p1, sb_data_atm & p2)
 	p2.ic1[2] = M_PI * p2.ic1[2] / 180.0;
 	
 	p1 >> buffer;
-	p2.bt[0] = bondtype(buffer[0]);
+	p2.bt = bondtype(buffer[0]);
 	
-	p1 >> p2.id[1] >> buffer;
-	p2.bt[1] = bondtype(buffer[0]);
+//////////////////////////////////////////////////////////////////////////////////////////
+if (p2.bt.GetValue() == 0) { cout << "bad bondtype A" << endl; exit(-1); }
+//////////////////////////////////////////////////////////////////////////////////////////
 	
 	return p1;
 }
@@ -895,7 +1098,7 @@ istream & operator>>(istream & p1, sb_data_atm & p2)
 ostream & operator<<(ostream & p1, sb_data_atm & p2)
 {
 	p1 << hex << "0x" << setw(4) << setfill('0') << p2.id << dec;
-	p1 << " " << p2.el.GetSymbol() << " " << p2.bt[0].GetSymbol1();
+	p1 << " " << p2.el.GetSymbol() << " " << p2.bt.GetSymbol1();
 	if (p2.tr != NULL) p1 << " " << (* p2.tr); p1 << " ";
 	
 	p1 << p2.ic1[0] << " " << p2.ic1[1] << " " << p2.ic2 << " " << p2.ic1[2] << " ";
@@ -922,6 +1125,11 @@ istream & operator>>(istream & p1, sb_data_bnd & p2)
 	while (p1.get() != ':');
 	p1 >> p2.atm[0] >> p2.atm[1];
 	p1 >> buffer; p2.bt = bondtype(buffer[0]);
+	
+//////////////////////////////////////////////////////////////////////////////////////////
+if (p2.bt.GetValue() == 0) { cout << "bad bondtype B" << endl; exit(-1); }
+//////////////////////////////////////////////////////////////////////////////////////////
+	
 	return p1;
 }
 
@@ -929,13 +1137,18 @@ istream & operator>>(istream & p1, sb_data_bnd & p2)
 
 sb_data_res::sb_data_res(void)
 {
+	symbol1 = '?';
+	strcpy(symbol3, "???");
+	
 	description = NULL;
 }
 
 sb_data_res::sb_data_res(const sb_data_res & p1)
 {
 	id = p1.id;
-	symbol = p1.symbol;
+	
+	symbol1 = p1.symbol1;
+	strcpy(symbol3, p1.symbol3);
 	
 	if (p1.description != NULL)
 	{
@@ -983,8 +1196,22 @@ void sb_data_res::ReadModification(istream & p1)
 istream & operator>>(istream & p1, sb_data_res & p2)
 {
 	char buffer[256];
-	while (p1.get() != 'S'); p1 >> p2.id;
-	while (p1.get() != ':'); p1 >> p2.symbol;
+	
+	while (p1.get() != 'S');
+	
+	p1 >> p2.id;
+	
+	while (p1.get() != ':');
+	
+	p1 >> buffer;
+	if (strlen(buffer) != 3)
+	{
+		cout << "bad symbol3 : " << buffer << endl;
+		exit(EXIT_FAILURE);
+	}
+	
+	strcpy(p2.symbol3, buffer);
+	p1 >> p2.symbol1;
 	
 	while (p1.get() != '"');
 	p1.getline(buffer, sizeof(buffer), '"');
