@@ -1,4 +1,4 @@
-// SEQBUILD.H : a sequencebuilder class for building and identifying sequences.
+// SEQBUILD.H : a sequence builder class for building and identifying sequences.
 
 // Copyright (C) 1998 Tommi Hassinen, Geoff Hutchison.
 
@@ -13,84 +13,61 @@
 #include "libghemicaldefine.h"
 //#include "libghemicalconfig2.h"
 
-class chn_info;
+class sb_chain_descriptor;
 
 class sequencebuilder;
 
-class sb_data_atm;
-class sb_data_bnd;
-class sb_data_res;
+class sb_data_atm;	// for internal use only...
+class sb_data_bnd;	// for internal use only...
 
-struct sb_tdata;
+class sb_data_res;	// for internal use only...
+
+struct sb_tdata;	// for internal use only...
 
 /*################################################################################################*/
 
 #include "atom.h"
 #include "bond.h"
 
-#include "model.h"
+#include "chn_info.h"
+
+//#include "model.h"
 #include "typerule.h"
 
 typedef vector<atom *> atmr_vector;
 
 /*################################################################################################*/
 
-#define PSTATE_CHARGE_mask		0x07	// three least significant bits...
-
-#define PSTATE_SIGN_NEGATIVE		0x08
-#define PSTATE_SIGN_POSITIVE		0x10
-#define PSTATE_CHARGED_TERMINAL		0x20
-
-/**	A "##chain info"-class. 
-	This is used in the context of peptides/proteins and nucleic acids...
+/**	A sb_chain_descriptor object describes the chain to be built for the Build() method of
+	the sequence builder class. It describes the chain using either 1-letter or 3-letter codes
+	and it can optionally contain information about 3D-structure as well.
 */
 
-class chn_info
+class sb_chain_descriptor
 {
-	public:
-	
-	enum chn_type
-	{
-		not_defined = 0, amino_acid = 1, nucleic_acid = 2
-	};
-	
 	protected:
 	
-	chn_type type;
+	bool mode_1_letter;
 	
-	i32s id_mol;		///< the molecule number (see also atom::id[]).
-	i32s id_chn;		///< the chain number (see also atom::id[]).
-	
-	i32s length;
-	char * sequence;
-	
-	char * ss_state;	// sec-str-state (as determined in DefineSecondaryStructure()).
-	char * p_state;		// protonation state (as determined in SF).
-	
-	char * description;
+	vector<char> * seq1;
+	vector<char *> * seq3;
 	
 	friend class sequencebuilder;
-	friend class setup1_sf;
-		
-	friend void DefineSecondaryStructure(model *);
-	friend f64 HBondEnergy(model *, i32s *, i32s *);
 	
 	public:
 	
-	chn_info(void);
-	chn_info(chn_type, i32s);
-	chn_info(const chn_info &);
-	~chn_info(void);
+	vector<f64> default_torsions;
+	
+	public:
+	
+	sb_chain_descriptor(bool);
+	~sb_chain_descriptor(void);
+	
+	int AddRes1(char);		// return value is the current chain length.
+	int AddRes3(const char *);	// return value is the current chain length.
 	
-	chn_type GetType(void) { return type; }
-	i32s GetLength(void) { return length; }
 	
-	i32s GetIDmol(void) { return id_mol; }
-	i32s GetIDchn(void) { return id_chn; }
 	
-	const char * GetSequence(void) { return sequence; }
-	const char * GetSecStrStates(void) { return ss_state; }
-	const char * GetProtonationStates(void) { return p_state; }
 };
 
 /*################################################################################################*/
@@ -114,7 +91,7 @@ class sequencebuilder
 	vector<sb_data_atm> main_vector;
 	vector<sb_data_atm> conn_vector;
 	
-	vector<sb_data_res> residue_vector;	// res_vector???
+	vector<sb_data_res> resi_vector;
 	
 	vector<typerule> head_vector;
 	vector<typerule> tail_vector;
@@ -137,17 +114,22 @@ class sequencebuilder
 	sequencebuilder(chn_info::chn_type, char *);
 	~sequencebuilder(void);
 	
-	void Build(model *, char *, f64 *);
+	void Build(model *, sb_chain_descriptor *);
+	
 	void Identify(model *);
 	
 	private:
 	
-	void Build(model *, sb_data_res *, f64 *);
+	// the builder
+	
+	void BuildResidue(model *, sb_data_res *, vector<f64> &);
 	void Convert(atom *[], f64 *, fGL *);
 	
-	void FindPath(model *, atom *, atom *, i32u = 0);
+	// the identifier
+	
 	void BuildTemplate(vector<sb_tdata> &, i32s, bool, bool);
-	void BuildTemplate(vector<sb_tdata> &, vector<sb_data_atm> &);
+	void BuildPartialT(vector<sb_tdata> &, vector<sb_data_atm> &);
+	void FindPath(model *, atom *, atom *, i32u = 0);
 	bool CheckTemplate(vector<sb_tdata> &, i32s);
 };
 
@@ -157,11 +139,11 @@ class sb_data_atm
 {
 	protected:
 	
-	i32s id[2];
+	i32s id;
 	i32s prev[3];
 	
 	element el;
-	bondtype bt[2];
+	bondtype bt;
 	
 	typerule * tr;
 	
@@ -207,7 +189,9 @@ class sb_data_res
 	
 	i32s id;
 	
-	char symbol;
+	char symbol1;		// a single-char symbol
+	char symbol3[4];	// a three-char string symbol
+	
 	char * description;
 	
 	vector<sb_data_atm> atm_vector;
