@@ -14,12 +14,10 @@
 
 #include <iomanip>
 #include <fstream>
-#include <strstream>
+#include <sstream>
 #include <algorithm>
 using namespace std;
 
-#define RAND_f64 ((f64) rand() / (f64) RAND_MAX - 0.5)
-
 /*################################################################################################*/
 
 prmfit_tables::prmfit_tables(const char * p1)
@@ -33,17 +31,18 @@ prmfit_tables::prmfit_tables(const char * p1)
 	ifstream file;
 	file.unsetf(ios::dec | ios::oct | ios::hex);
 	
-	char fn[1024];
+	ostringstream fns;
 	char buffer[1024];
 	
 /*##############################################*/
 /*##############################################*/
 
-	ostrstream fn_strt(fn, sizeof(fn));
-	fn_strt << path << "/atomtypes.txt" << ends;
-	file.open(fn, ios::in);
+	fns.str("");	// blank the filename!
+	fns << path << "/atomtypes.txt" << ends;
 	
-	if (ostr != NULL) (* ostr) << "reading file \"" << fn << "\": ";
+	file.open(fns.str().c_str(), ios::in);
+	
+	if (ostr != NULL) (* ostr) << "reading file \"" << fns.str() << "\": ";
 	
 	while (file.peek() != '#')		// #end
 	{
@@ -78,11 +77,12 @@ prmfit_tables::prmfit_tables(const char * p1)
 /*##############################################*/
 /*##############################################*/
 	
-	ostrstream fn_str1(fn, sizeof(fn));
-	fn_str1 << path << "/parameters1.txt" << ends;
-	file.open(fn, ios::in);
+	fns.str("");	// blank the filename!
+	fns << path << "/parameters1.txt" << ends;
 	
-	if (ostr != NULL) (* ostr) << "reading file \"" << fn << "\": ";
+	file.open(fns.str().c_str(), ios::in);
+	
+	if (ostr != NULL) (* ostr) << "reading file \"" << fns.str() << "\": ";
 	
 	while (file.peek() != '#')		// #end
 	{
@@ -106,11 +106,12 @@ prmfit_tables::prmfit_tables(const char * p1)
 /*##############################################*/
 /*##############################################*/
 	
-	ostrstream fn_str2(fn, sizeof(fn));
-	fn_str2 << path << "/parameters2.txt" << ends;
-	file.open(fn, ios::in);
+	fns.str("");	// blank the filename!
+	fns << path << "/parameters2.txt" << ends;
 	
-	if (ostr != NULL) (* ostr) << "reading file \"" << fn << "\": ";
+	file.open(fns.str().c_str(), ios::in);
+	
+	if (ostr != NULL) (* ostr) << "reading file \"" << fns.str() << "\": ";
 	
 	while (file.peek() != '#')		// #end
 	{
@@ -134,11 +135,12 @@ prmfit_tables::prmfit_tables(const char * p1)
 /*##############################################*/
 /*##############################################*/
 	
-	ostrstream fn_str3(fn, sizeof(fn));
-	fn_str3 << path << "/parameters3.txt" << ends;
-	file.open(fn, ios::in);
+	fns.str("");	// blank the filename!
+	fns << path << "/parameters3.txt" << ends;
 	
-	if (ostr != NULL) (* ostr) << "reading file \"" << fn << "\": ";
+	file.open(fns.str().c_str(), ios::in);
+	
+	if (ostr != NULL) (* ostr) << "reading file \"" << fns.str() << "\": ";
 	
 	while (file.peek() != '#')		// #end
 	{
@@ -162,11 +164,12 @@ prmfit_tables::prmfit_tables(const char * p1)
 /*##############################################*/
 /*##############################################*/
 
-	ostrstream fn_str4(fn, sizeof(fn));
-	fn_str4 << path << "/parameters4.txt" << ends;
-	file.open(fn, ios::in);
+	fns.str("");	// blank the filename!
+	fns << path << "/parameters4.txt" << ends;
 	
-	if (ostr != NULL) (* ostr) << "reading file \"" << fn << "\": ";
+	file.open(fns.str().c_str(), ios::in);
+	
+	if (ostr != NULL) (* ostr) << "reading file \"" << fns.str() << "\": ";
 	
 	while (file.peek() != '#')		// #end
 	{
@@ -189,7 +192,8 @@ prmfit_tables::prmfit_tables(const char * p1)
 	
 /*##############################################*/
 /*##############################################*/
-
+	
+	fns.str("");	// blank the filename!
 }
 
 prmfit_tables::~prmfit_tables(void)
@@ -265,15 +269,14 @@ i32u prmfit_tables::UpdateTypes(setup * su)
 		}
 		else
 		{
-			char mbuff1[256];
-			ostrstream str1(mbuff1, sizeof(mbuff1));
+			ostringstream str1;
 			str1 << "WARNING : could not determine atomtype (atom index = " << (* it1).index << ")." << endl << ends;
-			mdl->PrintToLog(mbuff1);
+			mdl->PrintToLog(str1.str().c_str());
 			
 			(* it1).atmtp = NOT_DEFINED;
 			(* it1).charge = 0.0;
 			
-			(* it1).flags |= ATOMFLAG_SELECTED;
+			(* it1).flags |= ATOMFLAG_USER_SELECTED;
 			errors++;
 		}
 	}
