@@ -12,12 +12,21 @@
 
 //#include "libghemicalconfig2.h"
 
+#include <math.h>
+
 /*################################################################################################*/
 
-/* added by Robert Williams for Compaq cxx, alpha 11/28/01 */
+// added by Robert Williams for Compaq cxx, alpha 11/28/01
+
 #define __USE_STD_IOSTREAM
 
-class engine;	// engine.h
+// the value for M_PI is needed for Win32 platform...
+
+#ifdef WIN32
+#define M_PI 3.141592654
+#endif	// WIN32
+
+class engine;	// engine.h ; this is for ValueFunction().
 
 #include <iostream>
 using namespace std;
@@ -29,10 +38,14 @@ typedef unsigned int i32u;
 
 typedef double f64;
 
-
 /*################################################################################################*/
 
 #ifdef HAVE_GL_GLUT_H
+
+#ifdef WIN32
+#include <windows.h>	// need to have this before the GL stuff...
+#endif	// WIN32
+
 #include <GL/glut.h>
 
 typedef GLint iGLs;
