@@ -459,7 +459,9 @@ void DefineSecondaryStructure(model * mdl)
 f64 HBondEnergy(model * mdl, i32s * res1, i32s * res2)
 {
 	vector<chn_info> & ci_vector = (* mdl->ref_civ);
-	if (ci_vector[res2[0]].sequence[res2[1]] == 'P') return 0.0;	// handle the PRO case...
+	
+	const char * seq = ci_vector[res2[0]].GetSequence1();
+	if (seq[res2[1]] == 'P') return 0.0;	// handle the PRO case...
 	
 	iter_al r1a[2]; mdl->GetRange(1, res1[0], r1a);
 	iter_al r1b[2]; mdl->GetRange(2, r1a, res1[1], r1b);
